/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.applicationRepository;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.LinkedList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import san.localization.Localization;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> BshAplPkg.java<br>
 * @author Michal Rejda
 * @since 13.10.2007
 * @version 1.0
 */
public class BshCodePkg extends CodePackage {

	private static final transient Logger log = Logger.getLogger(BshCodePkg.class);
	
	public static final String MANIFEST_ENTRY = "MANIFEST.MF";
	public static final String MANIFEST_PROPERTY_SEPARATOR = ":";
	public static final String BIN_EXTENSION = "java";
	public static final String CODE_PKG_CLOSED_EXCEPTION = Localization
			.getLocalization().getString("M000005");
	private ZipFile aplPkgZipFile = null;

	public BshCodePkg(CodePackageSymLink symLink, File aplPkgFile)
			throws AplPkgException {
		super(symLink, aplPkgFile);
		
		assert aplPkgFile != null;
		
		try {
			aplPkgZipFile = new ZipFile(aplPkgFile);
		}
		catch (ZipException e) {
			throw new AplPkgException(e.getMessage());
		}
		catch (IOException e) {
			throw new AplPkgException(e.getMessage());
		}
	}

	@Override
	public String[] getAllFullClassNamesFromPackage() throws AplPkgException {
		String capsuleMainClass = getManifestPropertyValue(CAPSULE_MAIN_CLASS_MFP);
		String applicationMainClass = getManifestPropertyValue(APP_MAIN_CLASS_MFP);
		
		Collection<String> classes = new LinkedList<String>();
		if(capsuleMainClass != null)
			classes.add(capsuleMainClass);
		if(applicationMainClass!= null)
			classes.add(applicationMainClass);
		
		return (String[]) classes.toArray();
	}

	@Override
	public InputStream getClass(String fullClassName)
			throws FileNotFoundException, AplPkgException {
		throw new FileNotFoundException(Localization.getLocalization()
				.getString("M000006"));
	}

	public InputStream getClass(String packageName, String className)
			throws FileNotFoundException {
		throw new FileNotFoundException(Localization.getLocalization()
				.getString("M000006"));
	}

	@Override
	public InputStream getApplicationMainClass() throws FileNotFoundException,
			AplPkgException {
		if (aplPkgZipFile == null)
			throw new AplPkgException(CODE_PKG_CLOSED_EXCEPTION);
		String mainClass = getManifestPropertyValue(APP_MAIN_CLASS_MFP);
		if (mainClass == null) {
			Object[] values = new Object[]{APP_MAIN_CLASS_MFP};
			throw new FileNotFoundException(Localization.getLocalization()
					.getString("M000007", values));
		}
		mainClass = mainClass.replaceAll("\\.", "/");
		try {
			ZipEntry ze = aplPkgZipFile.getEntry(mainClass + "."
					+ BIN_EXTENSION);
			return aplPkgZipFile.getInputStream(ze);
		}
		catch (IOException e) {
			Object[] values = new Object[]{mainClass, aplPkgFile.getName()};
			throw new FileNotFoundException(Localization.getLocalization()
					.getString("M000008", values));
		}
	}

	@Override
	public InputStream getCapsuleMainClass() throws FileNotFoundException,
			AplPkgException {
		if (aplPkgZipFile == null)
			throw new AplPkgException(CODE_PKG_CLOSED_EXCEPTION);
		String mainClass = getManifestPropertyValue(CAPSULE_MAIN_CLASS_MFP);
		if (mainClass == null) {
			Object[] values = new Object[]{CAPSULE_MAIN_CLASS_MFP};
			throw new FileNotFoundException(Localization.getLocalization()
					.getString("M000007", values));
		}
		mainClass = mainClass.replaceAll("\\.", "/");
		try {
			ZipEntry ze = aplPkgZipFile.getEntry(mainClass + "."
					+ BIN_EXTENSION);
			return aplPkgZipFile.getInputStream(ze);
		}
		catch (IOException e) {
			Object[] values = new Object[]{mainClass, aplPkgFile.getName()};
			throw new FileNotFoundException(Localization.getLocalization()
					.getString("M000009", values));
		}
	}

	@Override
	public String getManifestPropertyValue(String property)
			throws AplPkgException {
		if (aplPkgZipFile == null)
			throw new AplPkgException(CODE_PKG_CLOSED_EXCEPTION);
		ZipEntry ze = aplPkgZipFile.getEntry(MANIFEST_ENTRY);
		if (ze == null)
			return null;
		InputStream is = null;
		InputStreamReader isr = null;
		BufferedReader br = null;
		try {
			is = aplPkgZipFile.getInputStream(ze);
			isr = new InputStreamReader(is);
			br = new BufferedReader(isr);
			String text = null;
			String value = null;
			while ((text = br.readLine()) != null) {
				if (text.startsWith(property)) {
					value = text.substring(text
							.indexOf(MANIFEST_PROPERTY_SEPARATOR) + 1);
				}
			}
			if (value != null)
				value = value.trim();
			return value;
		}
		catch (Exception e) {
			
			return null;
		}
		finally{
			try {
				if (br != null)
					br.close();
			}
			catch (Exception ex) {
			}
			try {
				if (isr != null)
					isr.close();
			}
			catch (Exception ex) {
			}
			try {
				if (is != null)
					is.close();
			}
			catch (Exception ex) {
			}
		}
	}

	@Override
	public void close() throws IOException {
		super.close();
		
		// TODO zavrit jen kdyz neni otevreny (opens = 0) !!!!
		
		if(aplPkgZipFile == null){
			if (log.isDebugEnabled()) {
				log.debug("aplPkgZipFile je null");
			}
		}
		aplPkgZipFile.close();		
		aplPkgZipFile = null;
	}
}
