package san.applicationRepository;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.apache.log4j.Logger;

import san.Utility;
import san.codeDistribution.strategies.CodeDistributionStrategy;
import san.core.Kernel;
import san.data.CodePkgData;
import san.data.TransmitCapsule;
import san.ident.CodePkgIdent;

/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
public class Repository implements IRepository {

	private static final transient Logger log = Logger.getLogger(Repository.class);

	private String repositoryDir = "codeRepository";
	private File repositoryDirFile;
	
	public static final String BSH_EXTENSION = "bsh";
	public static final String SAN_EXTENSION = "spk";
	
	private int originalAppsCount = 0;
	private int maxNewAppsCount = 2;
	private int reqSent = 0;	//number of sent requests for an app
	
	private Map<CodePkgIdent, CodePackageSymLink> codePackageIndetToSymLinkMap;
	private Map<String, CodePackageSymLink> codePackageNameToSymLinkMap;
	private Map<CodePackageSymLink, CodePackage> symLinkToCodePackageMap;
	
	private Map<CodePkgIdent, Boolean> protectedApps;
	private Queue<TransmitCapsule> capsulesWaitingForApp;

	private Object repositoryLock = new Object();

	private CodeDistributionStrategy codeDistributionStrategy;

	private Kernel kernel;

	
	public Repository(Kernel kernel){
		this.kernel = kernel;
	}
	
	
	/*
	 * Initializes a repository and saves all apps from local directory in it
	 */
	public void initRepository() {
		
		codePackageNameToSymLinkMap = new ConcurrentHashMap<String, CodePackageSymLink>();
		codePackageIndetToSymLinkMap = new ConcurrentHashMap<CodePkgIdent, CodePackageSymLink>();
		symLinkToCodePackageMap = new ConcurrentHashMap<CodePackageSymLink, CodePackage>();
		
		protectedApps = new ConcurrentHashMap<CodePkgIdent, Boolean>();
		capsulesWaitingForApp = new ConcurrentLinkedQueue<TransmitCapsule>();
		
		try {
			// create permit extensions
			Collection<String> extensions = new Vector<String>();
			extensions.add(BSH_EXTENSION);
			extensions.add(SAN_EXTENSION);
			FilenameFilter fileFilter = new ExtensionsFilter(extensions);
			
			// list files with permit extensions
			repositoryDirFile = new File(repositoryDir);
			if(!repositoryDirFile.exists()){
				log.debug("Repository dir " + repositoryDir + " doesn't exists.");
				return;
			}
			File[] files = repositoryDirFile.listFiles(fileFilter);					
			
			for (int i = 0; i < files.length; i++) {
				
				CodePackageSymLink symlink = getCodePackageSymLinkFromLocalFile(files[i]);
				symlink.setNumberOfOpens(1);				
				
				if (symlink != null) {
					// add package into maps					
					addSymLinkIntoMaps(symlink);			
					
					if(symlink.getFileExtension().compareTo(SAN_EXTENSION) == 0 ){

						CodePackage cp = new SanCodePkg(symlink, files[i]);
						cp.setPkgIdentification(symlink.getPkgIdentification());
						cp.setPkgName(symlink.getPkgName());
						cp.setSince(symlink.getSince());
						
						symLinkToCodePackageMap.put(symlink, cp);
						
						protectedApps.put(cp.getPkgIdentification(), true);
					}
					else if(symlink.getFileExtension().compareTo(BSH_EXTENSION) == 0) {						
						CodePackage cp = new BshCodePkg(symlink, files[i]);
						cp.setPkgIdentification(symlink.getPkgIdentification());
						cp.setPkgName(symlink.getPkgName());
						cp.setSince(symlink.getSince());
						
						symLinkToCodePackageMap.put(symlink, cp);
						
						protectedApps.put(cp.getPkgIdentification(), true);
					}
					else{
						log.error("neznamy package, symlink neni vytvoren ani ulozen!");	
					}
				}
				else{
					log.error("nepodarilo se vytvorit symlink pro " + files[i] );
				}
			}
			
			originalAppsCount = codePackageIndetToSymLinkMap.size();
			
			log.debug("originaly apps count " + originalAppsCount);
			log.debug("max total apps count " + (originalAppsCount + maxNewAppsCount));

		}
		catch (Exception e) {
			log.error(e, e);
		}
	}
	
	/*
	 * Signals that app is no longer needed 
	 */
	@Override
	public void dontProtectAppFromDeleteAnyLonger(CodePkgIdent i) {
		
		synchronized (repositoryLock) {
			
			if (log.isTraceEnabled()) {
				log.trace("dontProtectAppFromDeleteAnyLonger(" + i + ")");
			}	
		
			
			CodePackageSymLink link = codePackageIndetToSymLinkMap.get(i);
			if (link == null) {
				log.error("code package " + i + " is not in the repository");
			}
			else{			
				// decrement number of opens, if it is zero, app can be removed from repository
				link.setNumberOfOpens(link.getNumberOfOpens() - 1);
				if (log.isDebugEnabled()) {
					log.debug("num of opens for " + i + " down to " + link.getNumberOfOpens());
				}
			}		
		}
	}
	
	/*
	 * Returns a code package based on identification
	 */
	@Override
	public CodePackage getCodePkg(CodePkgIdent i) {
		
		synchronized (repositoryLock) {
			
			if (log.isTraceEnabled()) {
				log.trace("getCodePkg(" + i + ")");
			}

			CodePackageSymLink link = codePackageIndetToSymLinkMap.get(i);
			if (link == null) {
				return null;
			}
			else{			
				// increment number of opens
				protectedApps.put(link.getPkgIdentification(), false);
										
				link.setNumberOfOpens(link.getNumberOfOpens() + 1);
				log.debug("num of opens for " + i + " up to " + link.getNumberOfOpens());

				return getCodePackage(link);
			}		
		}
	}

	/*
	 * Returns a code package based on name
	 */
	@Override
	public CodePackage getCodePkg(String name) {
		
		synchronized (repositoryLock) {
		
			if (log.isTraceEnabled()) {
				log.trace("getCodePkg(" + name + ")");
			}
		
			CodePackageSymLink link = codePackageNameToSymLinkMap.get(name);
			if (link == null){
				return null;
			}
			else{			
			
				//increment number of opens
				protectedApps.put(link.getPkgIdentification(), false);					
				link.setNumberOfOpens(link.getNumberOfOpens() + 1);
				log.debug("num of opens for " + name + " up to " + link.getNumberOfOpens());

				return getCodePackage(link);
			}
		}
	}

	
	/*
	 * Saves parameters as a code package in repository
	 */
	@Override
	public void saveCodePkg(CodePkgIdent hash, byte[] data, String extension, boolean fromFile) {

		synchronized (repositoryLock) {
			
			if (log.isTraceEnabled()) {
				log.trace("saveCodePkg(" + hash + ", " + data + ", " + extension + ")");
			}
			
			if(!fromFile){	//data are not from local directory
				reqSent--;
			}
			
			if(codePackageIndetToSymLinkMap.get(hash) != null){
				if (log.isDebugEnabled()) {
					log.debug("app " + hash + " is already in repository");
				}
				return;
			}
			
			//makes a space for an app
			if( codePackageIndetToSymLinkMap.size() >= (originalAppsCount + maxNewAppsCount) ){
				clearRepository();
			}		
			
			//still not enough space
			if( codePackageIndetToSymLinkMap.size()  >= (originalAppsCount + maxNewAppsCount) ){
				log.debug("canot save app " + hash + ", repository full ( " + codePackageIndetToSymLinkMap.size() + " apps)");
				protectedApps.put(hash, false);
				return;
			}
			
			//first we save data in temporary file 
			String tempFileName = repositoryDir + File.separator + hash.toString() + "." + extension;
			String destinationFileName = null;
			File f = null;
			FileOutputStream fos = null;

			try {

				fos = new FileOutputStream(tempFileName);
				fos.write(data);
				fos.flush();
				fos.close();

				f = new File(tempFileName);
				CodePackageSymLink symLink = new CodePackageSymLink();

				synchronized (symLink) {

					CodePackage cp = getCodePackageFromTempFile(f, symLink);

					if (cp != null) {

						symLink.setFileExtension(cp.getExtension());
						symLink.setPkgIdentification(cp.getPkgIdentification());
						symLink.setPkgLocation(tempFileName);
						symLink.setSince(System.currentTimeMillis());

						String pkgName = cp.getPkgName();

						if (pkgName != null && pkgName.length() > 0) {

							destinationFileName = repositoryDir
									+ File.separator + pkgName + "_"
									+ cp.getPkgIdentification() + "."
									+ extension;
							// close code package for rename
							cp.close();
							cp = null;
							// rename code package file
							if (f.renameTo(new File(destinationFileName)) == true) {
								if (log.isDebugEnabled()) {
									log.debug("podarilo se prejmenovat " + f.getName() + " -> "	+ destinationFileName);
								}
								// change code package file location
								symLink.setPkgLocation(destinationFileName);
								// set package name
								symLink.setPkgName(pkgName);
							} else {
								log.error("NEpodarilo se prejmenovat " + f.getName() + " -> " + destinationFileName);
							}
						} else {
							log.error("code package " + cp + " doesnt have a name, not added in map");
						}

						// adding packages symbolic link into maps
						symLink.setNumberOfOpens(0);
						log.debug("num of opens for " + symLink.getPkgIdentification() + " set to 0 after save");
						
						addSymLinkIntoMaps(symLink);

					} else {
						log.error("cannot create code package from temp file " + tempFileName);
					}
				}
			} catch (Exception e) {
				log.error("Exception during saving program: " + hash, e);

				try {
					if (fos != null) {
						fos.close();
					}
				} catch (Exception ex) {
					// nothing to do
				}
				if (f != null && f.exists()) {
					f.delete();
				}
			}

		}
	}

	
	/*
	 * Requests for applications are put in queue, here we try to send first one from queue
	 */
	@Override
	public void tryToSendNextRequestForApp() {
		
		synchronized (repositoryLock) {
			
			if (log.isTraceEnabled()) {
				log.trace("tryToSendNextRequestForApp()");
			}	
						
			if( (codePackageIndetToSymLinkMap.size() + reqSent)   >= (originalAppsCount + maxNewAppsCount) ){
				clearRepository();
			}		
			
			if( (codePackageIndetToSymLinkMap.size() + reqSent)  >= (originalAppsCount + maxNewAppsCount) ){
				log.debug("canot send app request, repository full ( " + codePackageIndetToSymLinkMap.size() + " apps + " + reqSent  + " requests for app sent)");
				return;
			}
			
			//request is send only if there is space for the app in repository
			
			if(capsulesWaitingForApp.size() > 0){
				
				if (log.isDebugEnabled()) {
					log.debug("sending req, apps in repository:  " + codePackageIndetToSymLinkMap.size());
				}
				
				reqSent++;
				
				codeDistributionStrategy.sendRequestForApp(capsulesWaitingForApp.poll());
			}
			else{
				log.debug("no capsules waiting for app");
			}
		}	
	}
	
	/*
	 * Put this requests for applications in queue, and try to send first one from queue
	 */
	@Override
	public void tryToSendNextRequestForApp(TransmitCapsule originalCapsule) {
		
		synchronized (repositoryLock) {
		
			if (log.isTraceEnabled()) {
				log.trace("tryToSendNextRequestForApp(" + originalCapsule + ")");
			}	
			
			if(codePackageIndetToSymLinkMap.get(originalCapsule.getHash()) != null){
				if (log.isDebugEnabled()) {
					log.debug("code for " + originalCapsule.getHash() + " is already in reposirory");
				}
				
				protectedApps.put(originalCapsule.getHash(), true);	
				kernel.codeStored(originalCapsule.getHash());
				return;
			}
			
			
			if(protectedApps.get(originalCapsule.getHash()) == null || protectedApps.get(originalCapsule.getHash()) == false){
				protectedApps.put(originalCapsule.getHash(), true);
				capsulesWaitingForApp.add(new TransmitCapsule(originalCapsule));
				tryToSendNextRequestForApp();
			}
			else{
				if (log.isDebugEnabled()) {
					log.debug("request for app " + originalCapsule.getHash() + " was already sent" );
				}
			}		
		}
	}
	
	/*
	 * Returns data from an code package or null 
	 */
	@Override
	public CodePkgData getCodePkgData(CodePkgIdent hash) {
		
		synchronized (repositoryLock) {
			
			if (log.isTraceEnabled()) {
				log.trace("getCodePkgData(" + hash + ")");
			}
			
			if(codePackageIndetToSymLinkMap.get(hash) == null){
				return null;
			}
			
			else{
				CodePackage cp = getCodePackage(codePackageIndetToSymLinkMap.get(hash));

				return new CodePkgData(cp.getPkgBytes(), cp.getExtension());
			}
		}
	}
	
	
	
	@Override
	public String getCodePkgName(CodePkgIdent i) {
		if(codePackageIndetToSymLinkMap.get(i) != null){
			return codePackageIndetToSymLinkMap.get(i).getPkgName();
		}
		
		return "unknown";		
	}
	
	@Override
	public Set<CodePkgIdent> getCpBaseOnPkgIdentificationMapKeys() {
		return codePackageIndetToSymLinkMap.keySet();
	}

	@Override
	public Set<String> getCpBaseOnPkgNamesMapKeys() {
		return codePackageNameToSymLinkMap.keySet();	
	}
	
	@Override
	public void setCodeDistributionStrategy(CodeDistributionStrategy codeDistributionStrategy) {
		this.codeDistributionStrategy = codeDistributionStrategy;

	}
	
	@Override
	public void setRepositoryDir(String repositoryDir) {
		
		if (log.isTraceEnabled()) {
			log.trace("setRepositoryDir(" + repositoryDir + ")");
		}
		
		this.repositoryDir = repositoryDir;
		initRepository();

	}
	
	@Override
	public boolean existCodePkg(CodePkgIdent ident) {
		return codePackageIndetToSymLinkMap.get(ident) != null;
	}

	/*
	 * Removes a code package from repository 
	 */
	private void removeCodePackage(CodePkgIdent cpi){
		
		if (log.isTraceEnabled()) {
			log.trace("removeCodePackage(" + cpi + ")");
		}
		
		CodePackageSymLink symLink = codePackageIndetToSymLinkMap.get(cpi); 
		String fileName = repositoryDir + File.separator + symLink.getPkgName() + "_"	+ symLink.getPkgIdentification() + "." + symLink.getFileExtension();
		
		try {
			if(symLinkToCodePackageMap.get(symLink) != null){
				symLinkToCodePackageMap.get(symLink).close();			
			}
		} catch (Exception e) {
			log.error("package " + symLink.getPkgName() + " cannot be closed", e);
		}
		
		codePackageNameToSymLinkMap.remove(symLink.getPkgName());
		codePackageIndetToSymLinkMap.remove(symLink.getPkgIdentification());
		symLinkToCodePackageMap.remove(symLink);	
		
		File f = new File(fileName);
		
		try {				
			f.delete();
		} catch (final Exception e) {
			log.error("file " + fileName + " cannot be deleted",   e);
		}		
	}
	
	/*
	 * Goes through all packages in repository and removes packages that are in use now
	 */
	private void clearRepository(){
		
		if (log.isTraceEnabled()) {
			log.trace("clearRepository()");
		}
		
		assert codePackageIndetToSymLinkMap.size() == codePackageNameToSymLinkMap.size();
		
		Collection<CodePkgIdent> toRemove = new LinkedList<CodePkgIdent>();
		for (CodePkgIdent cpi : codePackageIndetToSymLinkMap.keySet()) {
			if(codePackageIndetToSymLinkMap.get(cpi).getNumberOfOpens() == 0 && (protectedApps.get(cpi) == null || protectedApps.get(cpi) == false) ){
				toRemove.add(cpi);
			}
		}
		
		for (CodePkgIdent cpi : toRemove) {
			removeCodePackage(cpi);
		}
	}
	
	/*
	 * Makes an package from file
	 */
	private CodePackage getCodePackageFromTempFile(File location,
			CodePackageSymLink symLink) {

		
		int identIndex = 0, extnsionIndex = 0;
		String name = location.getName(); // get file name
		identIndex = name.lastIndexOf("_");
		extnsionIndex = name.lastIndexOf(".");

		if (identIndex < 0) { // get identification
			identIndex = -1;
		}
		String identification = name.substring(identIndex + 1, extnsionIndex);
		String extension = name.substring(extnsionIndex + 1);// get extension

		CodePackage aplPkg = null;

		try {
			// create package
			if (extension.compareToIgnoreCase(BSH_EXTENSION) == 0) {
				aplPkg = new BshCodePkg(symLink, location);

			} else if (extension.compareToIgnoreCase(SAN_EXTENSION) == 0) {
				aplPkg = new SanCodePkg(symLink, location);

			}

		} catch (AplPkgException e) {
			log.error(e.getMessage(), e);
		}

		if (aplPkg != null) {

			aplPkg.setPkgIdentification(new CodePkgIdent(Utility
					.fromHexString(identification)));
			aplPkg.setSince(location.lastModified());

			try {
				aplPkg.setPkgName(aplPkg.getManifestPropertyValue(CodePackage.PACKAGE_NAME_MFP));
			} catch (AplPkgException e) {
				log.error(e.getMessage(), e);
			}
		} else {
			log.error("aplication package is null");
		}

		return aplPkg;
	}

	/*
	 * Saves informations about packages in maps 
	 */
	private void addSymLinkIntoMaps(CodePackageSymLink symLink) {
		
		if (log.isTraceEnabled()) {
			log.trace("addSymLinkIntoMaps(" + symLink.getPkgName() + ")");
		}
		
		if (symLink == null) {
			return;
		}

		// codePackageNameToSymLinkMap
		if (symLink.getPkgName() != null) {
			CodePackageSymLink old = codePackageNameToSymLinkMap.get(symLink
					.getPkgName());

			// set latest code package in code package map
			if (old != null) {
				if (old.getSince() < symLink.getSince()) {
					codePackageNameToSymLinkMap.get(symLink.getPkgName()).setSince(symLink.getSince());
//					codePackageNameToSymLinkMap.put(symLink.getPkgName(), symLink);
				}
			} else {
				codePackageNameToSymLinkMap.put(symLink.getPkgName(), symLink);
			}
		} else {
			log.error("SymLink " + symLink	+ " doesnt have a name, not added in map");
		}

		// codePackageIndetToSymLinkMap
		if (symLink.getPkgIdentification() != null) {
			codePackageIndetToSymLinkMap.put(symLink.getPkgIdentification(), symLink);
		} else {
			log.error("SymLink " + symLink	+ " doesnt have a identification, not added in map");
		}
	}
	
	
	private CodePackage getCodePackage(CodePackageSymLink symLink) {
		CodePackage aplPkg = null;
		try {
			
			if (symLink.getFileExtension().compareToIgnoreCase(BSH_EXTENSION) == 0) {
				if(symLinkToCodePackageMap.get(symLink) == null){
					aplPkg = new BshCodePkg(symLink, new File(symLink.getPkgLocation()));
					symLinkToCodePackageMap.put(symLink, aplPkg);
				}
				else{
					aplPkg =symLinkToCodePackageMap.get(symLink);
					try{
						aplPkg.getApplicationMainClass();
					}catch (AplPkgException e) {
						aplPkg = new BshCodePkg(symLink, new File(symLink.getPkgLocation()));
						symLinkToCodePackageMap.put(symLink, aplPkg);
					} catch (FileNotFoundException e) {
						log.error(e.getMessage(), e);
					}
				}
				
			} else if (symLink.getFileExtension().compareToIgnoreCase(SAN_EXTENSION) == 0) {
				if(symLinkToCodePackageMap.get(symLink) == null){
					aplPkg = new SanCodePkg(symLink, new File(symLink.getPkgLocation()));
					symLinkToCodePackageMap.put(symLink, aplPkg);
				}
				else{
					aplPkg = symLinkToCodePackageMap.get(symLink);
					try{
						aplPkg.getApplicationMainClass();
					}catch (AplPkgException e) {
						aplPkg = new SanCodePkg(symLink, new File(symLink.getPkgLocation()));
						symLinkToCodePackageMap.put(symLink, aplPkg);
					} catch (FileNotFoundException e) {
						log.error(e.getMessage(), e);
					}
				}
			}
		}
		catch (AplPkgException e) {
			log.error(e.getMessage(), e);
		}
		if (aplPkg != null) {
			
			aplPkg.setPkgIdentification(symLink.getPkgIdentification());			
			aplPkg.setSince(symLink.getSince());			
			aplPkg.setPkgName(symLink.getPkgName());
		}
		else{
			log.error("aplication package is null");
		}
		return aplPkg;
	}

	private CodePackageSymLink getCodePackageSymLinkFromLocalFile(File file) {
		
		if (file == null){
			return null;
		}
		
		int identIndex = 0, extnsionIndex = 0;
		String name = file.getName();
		identIndex = name.lastIndexOf("_");
		extnsionIndex = name.lastIndexOf(".");
		
		// get identification
		if (identIndex < 0){
			identIndex = -1;
		}
		
		// get pkg name
		String pkgName = null;
		if (identIndex > 0){
			pkgName = name.substring(0, identIndex);
		}
		
		// get identification
		String identification = name.substring(identIndex + 1, extnsionIndex);
		// get extension
		String extension = name.substring(extnsionIndex + 1);
		// creating code package symbolic link
		CodePackageSymLink symlink = new CodePackageSymLink();
		symlink.setPkgIdentification(new CodePkgIdent(Utility.fromHexString(identification)));
		symlink.setPkgName(pkgName);
		symlink.setPkgLocation(file.getAbsolutePath());
		symlink.setSince(file.lastModified());
		symlink.setFileExtension(extension);
		
		return symlink;
	}
	
	
	private class ExtensionsFilter implements FilenameFilter {

		private Collection<String> permitExtensions;

		public ExtensionsFilter(Collection<String> permitExtensions) {
			this.permitExtensions = permitExtensions;
		}

		public boolean accept(File dir, String name) {
			int lastDotIndex = name.lastIndexOf(".");
			if (lastDotIndex < 0)
				return false;
			name = name.substring(lastDotIndex + 1);
			return permitExtensions.contains(name);
		}
	}

	public int getMaxNewAppsCount() {
		return maxNewAppsCount;
	}
	
	public void setMaxNewAppsCount(int maxNewAppsCount) {
		this.maxNewAppsCount = maxNewAppsCount;
	}
}
