/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.codeDistribution;

import java.util.Hashtable;
import java.util.Map;

import org.apache.log4j.Logger;

import san.applicationRepository.IRepository;
import san.core.CapsuleAddedInQueueListener;
import san.core.CodeStoredListener;
import san.core.Kernel;
import san.data.TransmitCapsule;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.networkInterfaces.NetInterface;
import san.networkInterfaces.NetInterfaces;
import san.outgoingQueue.OutgoingQueue;
import san.routing.RoutingRecord;
import san.routing.RoutingTable;

/*
 * Records traffic on this node, all capsules sent or recieved
 */
public class CapsuleTrafficLogger implements CodeStoredListener,
		CapsuleAddedInQueueListener {

	private static final transient Logger log = Logger
			.getLogger(CapsuleTrafficLogger.class);
	
	private IRepository repository;
	private OutgoingQueue outQueue;
	private Map<CodePkgIdent, AppPlacementInfo> appPlacementLog = new Hashtable<CodePkgIdent, AppPlacementInfo>();
	private Map<NetIdentifier, CapsuleTrafficLoggerData> capsuleTrafficLog = new Hashtable<NetIdentifier, CapsuleTrafficLoggerData>();
	private NetInterfaces netInterfaces;
	private Kernel kernel;
	
	private long dataCapsulesSentCount = 0;
	private long appCapsulesSentCount = 0;
	private long appReqCapsulesSentCount = 0;
	private long dataCapsulesForwardCount = 0;
	private long appCapsulesForwardCount = 0;
	private long appReqCapsulesForwardCount = 0;

	public CapsuleTrafficLogger(Kernel kernel) {
		super();

		this.kernel = kernel;
		this.repository = kernel.getRepository();
		this.outQueue = kernel.getOutQueue();

		this.netInterfaces = kernel.getNetInterfaces();

		for (CodePkgIdent cpi : kernel.getRepository().getCpBaseOnPkgIdentificationMapKeys()) {
			AppPlacementInfo capsuleInfo;

			if (appPlacementLog.containsKey(cpi)) {
				capsuleInfo = appPlacementLog.get(cpi);
			} else {
				capsuleInfo = new AppPlacementInfo(kernel);
				appPlacementLog.put(cpi, capsuleInfo);
			}
			capsuleInfo.setStoredLocaly(true);
		}
	}
	
	/*
	 * Saves informations about apps in repository
	 */
	public void rereadLocalApps(){
		for (CodePkgIdent cpi : kernel.getRepository()
				.getCpBaseOnPkgIdentificationMapKeys()) {
			AppPlacementInfo capsuleInfo;

			if (appPlacementLog.containsKey(cpi)) {
				capsuleInfo = appPlacementLog.get(cpi);
			} else {
				capsuleInfo = new AppPlacementInfo(kernel);
				appPlacementLog.put(cpi, capsuleInfo);
			}
			capsuleInfo.setStoredLocaly(true);
		}
	}
	
	/*
	 * Saves parameters of one capsule
	 */
	private synchronized void logCapsule(TransmitCapsule capsule) {
		AppPlacementInfo capsuleInfo;

		if (appPlacementLog.containsKey(capsule.getHash())) {
			capsuleInfo = appPlacementLog.get(capsule.getHash());
		} else {
			capsuleInfo = new AppPlacementInfo(kernel);
			appPlacementLog.put(capsule.getHash(), capsuleInfo);
		}

		if (capsule.isAplication()) {
			capsuleInfo.addNodeAppCameFrom(capsule.getSrc());
			capsuleInfo.addNodeAppCameFrom(capsule.getPrev());

			capsuleInfo.addNodeAppSentTo(capsule.getDest());
			capsuleInfo.addNodeAppSentTo(capsule.getOver());
			
		} else if (capsule.isRequestForApplication()) {
			capsuleInfo.removeNodeAppCameFrom(capsule.getSrc());
			capsuleInfo.removeNodeAppCameFrom(capsule.getPrev());
			
		} else {
			capsuleInfo.addNodeDataCameFrom(capsule.getSrc());
			capsuleInfo.addNodeDataCameFrom(capsule.getPrev());

			capsuleInfo.addNodeDataSentTo(capsule.getDest());
			capsuleInfo.addNodeDataSentTo(capsule.getOver());
				
		}

	}

	/*
	 * Return source code of a webpage with collected informations and informations
	 * about the whole node as well
	 */
	public String getAppPlacementLogInHtml() {

		if (log.isTraceEnabled()) {
			log.trace("getAppPlacementLogInHtml()");
		}
		
		StringBuilder html = new StringBuilder();
		html.append("<html>");

		
		/*
		 * capsules counts
		 * -------------
		 */
		html.append("<h1>Capsule statistics</h1>");
		
		html.append("<table cellPadding=\"5px\" border=\"1px solid black\">");
		html.append("<tr style=\"background: #A8A8A8; color: white;\">");
		html.append("<th>type</th><th>sent</th><th>forward</th>");
		html.append("</tr>");

		html.append("<tr>");
		html.append("<td>data</td><td>" + dataCapsulesSentCount	+ "</td><td>" + dataCapsulesForwardCount	+ "</td>");
		html.append("</tr>");

		html.append("<tr>");
		html.append("<td>app</td><td>" + appCapsulesSentCount	+ "</td><td>" + appCapsulesForwardCount	+ "</td>");
		html.append("</tr>");

		html.append("<tr>");
		html.append("<td>req4app</td><td>"+ appReqCapsulesSentCount	+ "</td><td>" + appReqCapsulesForwardCount	+ "</td>");
		html.append("</tr>");
		html.append("</table></p>");

		
		/*
		 * details about sent capsules
		 * -----------------------------------------------
		 */
		html.append("<h1>Capsules sent between nodes</h1>");
		
		html.append("<table cellPadding=\"5px\" border=\"1px solid black\">");
		html.append("<tr style=\"background: #A8A8A8; color: white;\">");		
		html.append("<th>Remote interf.</th>");
		html.append("<th>type</th>");
		html.append("<th>sent</th>");
		html.append("<th>forward</th>");		
		html.append("</tr>");

		for (NetIdentifier n : capsuleTrafficLog.keySet()) {			

			CapsuleTrafficLoggerData data = capsuleTrafficLog.get(n);
			
			html.append("<tr>");
			html.append("<td rowspan=\"3\">[" + n + "]</td>");
			html.append("<td >data</td>");
			html.append("<td >" + data.getDataCapsulesSentCount() + "</td>");
			html.append("<td >" + data.getDataCapsulesForwardCount() + "</td>");
			html.append("</tr>");
			
			html.append("<tr>");
			html.append("<td >app</td>");
			html.append("<td >" + data.getAppCapsulesSentCount() + "</td>");
			html.append("<td >" + data.getAppCapsulesForwardCount() + "</td>");
			html.append("</tr>");
			
			html.append("<tr>");			
			html.append("<td >req4app</td>");
			html.append("<td >" + data.getAppReqCapsulesSentCount() + "</td>");
			html.append("<td >" + data.getAppReqCapsulesForwardCount() + "</td>");
			html.append("</tr>");
						
		}

		html.append("</table></p>");
	
		
		/*
		 * routing information
		 * --------------
		 */
		html.append("<h1>Routing table</h1>");
	
		html.append("<table cellPadding=\"5px\" border=\"1px solid black\">");
		html.append("<tr style=\"background: #A8A8A8; color: white;\">");
		html.append("<th>dest</th><th>rout info</th>");
		html.append("</tr>");
		
		RoutingTable rt = kernel.getRouter().getRoutingTable();
		for(NetIdentifier ni : rt.getRoutingRecords().keySet()){
			html.append("<tr>");
				html.append("<td> [" + ni+ "]</td>");
				html.append("<td>"); 
				for(RoutingRecord rr : rt.getRoutingRecords().get(ni)){
					html.append("interface: [" + rr.getNetInterface() + "], gateway: [" + rr.getGateway() + "]<br/>");
				}
				html.append("</td>");
			html.append("</tr>");
		}
		
		html.append("<tr style=\"background: #A8A8A8; color: white;\">");
		html.append("<th>network</th><th>default route</th>");
		
		
		for(Guid guid : kernel.getRouter().getDefaultGateways().keySet()){
			html.append("<tr>");
			html.append("<td>" + guid + "</td>");
			html.append("<td>interface: [" + kernel.getRouter().getDefaultGateways().get(guid).getNetInterface() 
					+ "], gateway: [" + kernel.getRouter().getDefaultGateways().get(guid).getGateway() + "]</td>"); 
			html.append("</tr>");
		}
		
		html.append("</table></p>");
		
		/*
		 * interfaces, connections
		 * -----------------------------------
		 */
		html.append("<h1>Local & remote interfaces</h1>");
		
		html.append("<table cellPadding=\"5px\" border=\"1px solid black\">");
		html.append("<tr style=\"background: #A8A8A8; color: white;\">");
		html.append("<th>local</th><th>remote</th><th>ID</th>");
		html.append("</tr>");

		for (NetIdentifier n : netInterfaces.getNetInterfaces().keySet()) {
			html.append("<tr>");
			html.append("<td>Local interface</td>");
			html.append("<td>&nbsp;&nbsp;</td>");
			html.append("<td> [" + n + "]</td>");
			html.append("</tr>");

			NetInterface ni = netInterfaces.getNetInterfaces().get(n);
			for (NetIdentifier r : ni.getRemoteInterfaces().keySet()) {

				html.append("<tr>");
				html.append("<td style=\"valign: left;\">&nbsp;&nbsp;&nbsp;---></td>");
				html.append("<td>Remote interface</td>");
				html.append("<td> [" + r + "]</td>");
				html.append("</tr>");
			}
		}

		html.append("</table></p>");
		
		/*
		 * apps info
		 * -----------
		 */
		html.append("<h1>App log</h1>");
		
		html.append("<table cellPadding=\"5px\" border=\"1px solid black\">");
		html.append("<tr style=\"background: #A8A8A8; color: white;\">");
		html
				.append("<th>application</th><th>hash</th><th>name</th><th>values</th>");
		html.append("</tr>");

		for (CodePkgIdent i : appPlacementLog.keySet()) {

			html.append("<tr>");
			html.append("<td rowspan=\"7\">" + repository.getCodePkgName(i) + "</td>");			
			html.append("</tr>");

			html.append("<tr>");
			html.append("<td rowspan=\"7\">" + i + "</td>");
			html.append("</tr>");

			html.append("<tr >");
			html.append("<td>Stored In Node</td>");
//			if (appPlacementLog.get(i).isStoredLocaly()) {
			if(repository.existCodePkg(i)){				
				html.append("<td style=\"background: #33CC33; color: white;\">true</td>");
			} else {
				html.append("<td style=\"background: #CC3366; color: white;\">false</td>");
			}
			html.append("</tr>");

			html.append("<tr>");
			html.append("<td>Nodes App Came From</td>");
			html.append("<td>");
			for (NetIdentifier ni : appPlacementLog.get(i)
					.getNodesAppCameFrom()) {
				html.append(" [" + ni + "], ");
			}
			html.append("</td>");
			html.append("</tr>");

			html.append("<tr>");
			html.append("<td>Nodes App Sent To</td>");
			html.append("<td>");
			for (NetIdentifier ni : appPlacementLog.get(i).getNodesAppSentTo()) {
				html.append(" [" + ni + "], ");
			}
			html.append("</td>");
			html.append("</tr>");

			html.append("<tr>");
			html.append("<td>Nodes Data Came From</td>");
			html.append("<td>");
			for (NetIdentifier ni : appPlacementLog.get(i)
					.getNodesDataCameFrom()) {
				html.append(" [" + ni + "], ");
			}
			html.append("</td>");
			html.append("</tr>");

			html.append("<tr>");
			html.append("<td>Nodes Data Sent To</td>");
			html.append("<td>");
			for (NetIdentifier ni : appPlacementLog.get(i).getNodesDataSentTo()) {
				html.append(" [" + ni + "], ");
			}
			html.append("</td>");
			html.append("</tr>");

		}

		html.append("</table>");
		
		html.append("</html>");

		return html.toString();
	}

	@Override
	public void capsuleAdded(TransmitCapsule capsule) {
		logCapsule(capsule);
	}

	@Override
	public void capsuleAdded(TransmitCapsule capsule, String name) {
		
		
		if(name.equals(OutgoingQueue.class.getName())){
			
			CapsuleTrafficLoggerData traffic = capsuleTrafficLog.get(capsule.getOver());
			if(traffic == null){
				traffic = new CapsuleTrafficLoggerData();
				capsuleTrafficLog.put(capsule.getOver(), traffic);
			}
			
			if (capsule.isAplication()) {
			
				if(kernel.isMyInterfaceOrNull(capsule.getSrc())){
					appCapsulesSentCount++;
					traffic.incrementAppCapsulesSentCount();
				}else{
					appCapsulesForwardCount++;
					traffic.incrementAppCapsulesForwardCount();
				}
			} else if (capsule.isRequestForApplication()) {
			
				if(kernel.isMyInterfaceOrNull(capsule.getSrc())){
					appReqCapsulesSentCount++;
					traffic.incrementAppReqCapsulesSentCount();
				}else{
					appReqCapsulesForwardCount++;
					traffic.incrementAppReqCapsulesForwardCount();
				}
			} else {
					
				if(kernel.isMyInterfaceOrNull(capsule.getSrc())){
					dataCapsulesSentCount++;
					traffic.incrementDataCapsulesSentCount();
				}else{
					dataCapsulesForwardCount++;
					traffic.incrementDataCapsulesForwardCount();
				}
			}
		}
		
		
		logCapsule(capsule);
	}

	@Override
	public void codeStored(CodePkgIdent hash) {		
		appPlacementLog.get(hash).setStoredLocaly(true);
	}

	@Override
	public void codeStored(TransmitCapsule capsule) {		
		appPlacementLog.get(capsule.getHash()).setStoredLocaly(true);
	}

	public Map<CodePkgIdent, AppPlacementInfo> getAppPlacementLog() {
		return appPlacementLog;
	}
	
	public AppPlacementInfo getAppPlacementInfo(CodePkgIdent i){
		return appPlacementLog.get(i);
	}
}
