/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Collection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.spi.LoggingEvent;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import san.applicationRepository.AplPkgException;
import san.applicationRepository.CodePackage;
import san.applicationRepository.IRepository;
import san.applicationRepository.Repository;
import san.codeDistribution.CapsuleTrafficLogger;
import san.codeDistribution.strategies.CodeDistributionStrategy;
import san.codeDistribution.strategies.NaiveCodeDistributionStrategy;
import san.console.IConsole;
import san.console.TextConsole;
import san.console.WindowConsoleContainer;
import san.control.ControlHttpServer;
import san.data.TransmitCapsule;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.ii.InterceptorInjector;
import san.incomingQueue.IncomingQueue;
import san.interpreter.InterpretManager;
import san.localization.Localization;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;
import san.logging.LoggerFactory.LogOutput;
import san.networkInterfaces.AccessibleMethod;
import san.networkInterfaces.NetInterface;
import san.networkInterfaces.NetInterfaces;
import san.outgoingQueue.OutgoingQueue;
import san.routing.IRoutingTableControl;
import san.routing.Router;
import san.softstate.ISoftState;
import san.softstate.SoftState;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Kernel of the node.<br>
 * 
 * @author Michal Rejda
 * @since 30.10.2006
 * @version 1.1
 * @uml.dependency supplier="san.applicationRepository.IRepository"
 */
public class Kernel implements IServerControl, CodeStoredListener {
 
	private static final transient Logger log = Logger.getLogger(Kernel.class);	
	
//	private static Kernel instance;
	/**
	 * File with server settings.
	 */
	public static final String SETTINGS_FILE = "settings.xml";
	/**
	 * @uml.property name="netInterfaces"
	 * @uml.associationEnd readOnly="true" multiplicity="(1 1)"
	 *                     inverse="kernel:san.networkInterfaces.NetInterfaces"
	 */
	private NetInterfaces netInterfaces = null;
	/**
	 * @uml.property name="inQueue"
	 * @uml.associationEnd readOnly="true" multiplicity="(1 1)"
	 *                     inverse="kernel:san.incomingQueue.IncomingQueue"
	 */
	private IncomingQueue inQueue = null;
	/**
	 * @uml.property name="outQueue"
	 * @uml.associationEnd readOnly="true" multiplicity="(1 1)"
	 *                     inverse="kernel:san.outgoingQueue.OutgoingQueue"
	 */
	private OutgoingQueue outQueue = null;
	/**
	 * @uml.property name="codeStoreManager"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="kernel:san.core.CodeStoreManager"
	 */
	private CodeStoreManager codeStoreManager = null;
	/**
	 * @uml.property name="repository"
	 * @uml.associationEnd readOnly="true" multiplicity="(1 1)"
	 *                     inverse="kernel:san.applicationRepository.IRepository"
	 */
	private IRepository repository = null;
	/**
	 * @uml.property name="router"
	 * @uml.associationEnd readOnly="true" multiplicity="(1 1)"
	 *                     inverse="kernel:san.routing.Router"
	 */
	private Router router = null;
	private ISoftState softState = null;
	private boolean textConsole = true;	
	private CapsuleTrafficLogger capsuleTrafficLogger;

	private CodeDistributionStrategy codeDistributionStrategy;

  /**
   * reference to interceptor/injector component
   */
  private InterceptorInjector interceptorInjector;

  private Guid initGuid;

	public Kernel() {
		this("");
	}

	public Kernel(String settingsFile) {

		softState = new SoftState();
		netInterfaces = new NetInterfaces();
		router = new Router(netInterfaces);
		outQueue = new OutgoingQueue(netInterfaces);
    interceptorInjector = InterceptorInjector.getInterceptorInjector(this);
		
		Repository repository = new Repository(this);
		repository.initRepository();
		this.repository = repository;
		capsuleTrafficLogger = new CapsuleTrafficLogger(this);
		this.codeDistributionStrategy = new NaiveCodeDistributionStrategy(this.repository, this.router, this, this.capsuleTrafficLogger);
//		this.codeDistributionStrategy = new AskNeighbourDistributionStrategy(this.repository, this.router, this, this.capsuleTrafficLogger);
		
		this.repository.setCodeDistributionStrategy(this.codeDistributionStrategy);
		inQueue = new IncomingQueue(this, netInterfaces, codeDistributionStrategy);	
		inQueue.addCapsuleAddedInQueueListener(this.capsuleTrafficLogger);
		outQueue.addCapsuleAddedInQueueListener(this.capsuleTrafficLogger);
		
		if(settingsFile == null || settingsFile.length() == 0){
			readSettings(SETTINGS_FILE);
		}
		else{
			readSettings(settingsFile);
		}		
		
		capsuleTrafficLogger.rereadLocalApps();
		
		interpretManager = new InterpretManager(this);
		scheduler = new Scheduler(this, repository, inQueue, interpretManager, codeDistributionStrategy);
		interpretManager.setScheduler(scheduler);
		
		codeStoreManager = new CodeStoreManager(inQueue, repository);
		codeStoreManager.addCodeStoredListener(this);
		codeStoreManager.addCodeStoredListener(this.capsuleTrafficLogger);		
		
		outQueue.start();
    
    interceptorInjector.start();
		
		try{
			Integer.parseInt(settingsFile.substring(settingsFile.length() - 5, settingsFile.length() - 4));
			new ControlHttpServer(this).startHttpServer(5560 + Integer.parseInt(settingsFile.substring(settingsFile.length() - 5, settingsFile.length() - 4)));
		}
		catch(NumberFormatException e){
			
		}
		

		// run initialize application
		try {
			initGuid = runApplication("init", null);
		}

		catch (ExecuteException e) {
			LoggerFactory.createLogger().Log(e, LogType.ERROR);
		}
	}
	
	/**
	 * Read config from file.
	 */
	private void readSettings(String file) {
		try {
			if (new File(file).exists() == true
					&& new File(file).isFile() == true) {
				DocumentBuilderFactory dbf = DocumentBuilderFactory
						.newInstance();
				dbf.setValidating(false);
				DocumentBuilder parser = dbf.newDocumentBuilder();
				Document doc = parser.parse(file);
				readSettings(doc); // nacteni nastaveni
			}
		} catch (Exception e) {
			LoggerFactory.createLogger().Log(
					Localization.getLocalization().getString("M000017"),
					LogType.ERROR);
		}
	}

	/**
	 * Read settings from document.
	 */
	private void readSettings(Document doc) {
		BaseSettings bs = BaseSettings.getSettings();
		NodeList nl;
		Node n;
		// read server name
		String serverName = doc.getDocumentElement().getAttribute("name");
		bs.setServerName(serverName);
		// read log
		nl = doc.getElementsByTagName("logging");
		n = nl.item(0);
		String level = ((Element) n).getAttribute("level");
		bs.setLogLevel(LogType.valueOf(level));
		LoggerFactory.logLevel = bs.getLogLevel();
		String logOutput = ((Element) n).getAttribute("output");
		bs.setLogOutput(LogOutput.valueOf(logOutput));
		LoggerFactory.logOutput = bs.getLogOutput();
		// select console type
		nl = doc.getElementsByTagName("console");
		n = nl.item(0);
		String consoleType = ((Element) n).getAttribute("type");
		if (consoleType.compareToIgnoreCase("graphic") == 0) {
			textConsole = false;
		} else {
			textConsole = true;
		}
				
		nl = doc.getElementsByTagName("maxNewAppsInRepositorycount");
		int maxNewAppsInRepositorycount = Integer.parseInt(getNodeText(nl.item(0)));
		repository.setMaxNewAppsCount(maxNewAppsInRepositorycount) ;

		nl = doc.getElementsByTagName("codeRepository");
		String codeRepository = getNodeText(nl.item(0));
		repository.setRepositoryDir(codeRepository);

		// read interfaces
		// NodeList interfacesNL = doc.getElementsByTagName("interface");
		Node node = doc.getDocumentElement();
		NodeList interfacesNL = ((Element) node).getChildNodes();
		String name;
		NetInterface netInterface;
		InetAddress ip;
		int port;
		NetworkInterface networkInt;
		NodeList nlMethod;
		Node interfaceN;
		Node nMethod;
		int method;
		int priority;
		String addString;
		String netString;
		Guid addGuid;
		Guid netGuid;
		for (int i = 0; i < interfacesNL.getLength(); i++) {
			try {
				interfaceN = interfacesNL.item(i);
				if (interfaceN.getNodeName().equals("interface") == false) {
					continue;
				}
				name = ((Element) interfaceN).getAttribute("name"); // name of
				// the
				// interface
				networkInt = NetworkInterface.getByName(name);
				// network identity
				nl = ((Element) interfaceN).getElementsByTagName("node"); // read
				// node
				// address
				addString = getNodeText(nl.item(0));
				addGuid = Guid.parseGuid(addString);
				nl = ((Element) interfaceN).getElementsByTagName("network");// read
				// node
				// network
				netString = getNodeText(nl.item(0));
				netGuid = Guid.parseGuid(netString);
				// broadcast settings
				n = getNodeWithName(interfaceN, "broadcast");
				port = Integer.parseInt(((Element) n).getAttribute("port")); // port
				// of
				// broadcast
				ip = InetAddress.getByName(((Element) n).getAttribute("ip")); // address
				// of
				// broadcast
				netInterface = new NetInterface(new NetIdentifier(addGuid,
						netGuid), networkInt, ip, port);
				// read number and priority
				nlMethod = ((Element) interfaceN)
						.getElementsByTagName("method"); // methods of access
				for (int j = 0; j < nlMethod.getLength(); j++) {
					nMethod = nlMethod.item(j);
					method = Integer.parseInt(((Element) nMethod)
							.getAttribute("value"));
					priority = Integer.parseInt(((Element) nMethod)
							.getAttribute("priority"));
					netInterface.addAccessibleMethod(new AccessibleMethod(
							priority, method));
				}
				// add the interface into interfaces
				netInterfaces.addNetInterface(netInterface);
				// add the interface to the router
				netInterface.start();
			} catch (UnknownHostException e) {
				LoggerFactory.createLogger().Log(e, LogType.ERROR);
			} catch (SocketException e) {
				Object[] values = new Object[] { e };
				LoggerFactory.createLogger().Log(
						Localization.getLocalization().getString("M000018",
								values), LogType.ERROR);
			}
		}

		//routing
		NodeList routingNL = doc.getElementsByTagName("route");
		for (int i = 0; i < routingNL.getLength(); i++) {
			Node routeN = routingNL.item(i);
			nl = ((Element) routeN).getElementsByTagName("to");
			Node toN = nl.item(0);
			nl = ((Element) routeN).getElementsByTagName("gateway");
			Node gatewayN = nl.item(0);
			nl = ((Element) routeN).getElementsByTagName("interface");
			interfaceN = nl.item(0);
			// network identity
			nl = ((Element) toN).getElementsByTagName("node"); // read
			// node address
			addString = getNodeText(nl.item(0));
			addGuid = Guid.parseGuid(addString);
			nl = ((Element) toN).getElementsByTagName("network");// read
			// node network
			netString = getNodeText(nl.item(0));
			netGuid = Guid.parseGuid(netString);

			NetIdentifier to = new NetIdentifier(addGuid, netGuid);

			// network identity
			nl = ((Element) gatewayN).getElementsByTagName("node"); // read
			// node address
			addString = getNodeText(nl.item(0));
			addGuid = Guid.parseGuid(addString);
			nl = ((Element) gatewayN).getElementsByTagName("network");// read
			// node network
			netString = getNodeText(nl.item(0));
			netGuid = Guid.parseGuid(netString);

			NetIdentifier gateway = new NetIdentifier(addGuid, netGuid);

			// network identity
			nl = ((Element) interfaceN).getElementsByTagName("node"); // read
			// node address
			addString = getNodeText(nl.item(0));
			addGuid = Guid.parseGuid(addString);
			nl = ((Element) interfaceN).getElementsByTagName("network");// read
			// node network
			netString = getNodeText(nl.item(0));
			netGuid = Guid.parseGuid(netString);

			NetIdentifier interfc = new NetIdentifier(addGuid, netGuid);

			router.getRoutingTable().add("Static route", to, gateway, 1,
					interfc);
		}
		
		//default gateways
		NodeList defGatewayNL = doc.getElementsByTagName("defaultGateways");
		if(defGatewayNL.getLength() == 0){
			log.error("No default gateway found in settings.");
		}
		else{
			
			Node defGatewayN = defGatewayNL.item(0);
			nl = ((Element) defGatewayN).getElementsByTagName("gateway");
			
			for(int i =0; i < nl.getLength(); i++){
				Node gatewayN = nl.item(i);
				String netAttrinute = ((Element) gatewayN).getAttribute("network");
				Guid net = Guid.parseGuid(netAttrinute);
				
				NodeList partNL = ((Element) gatewayN).getElementsByTagName("identity");
				Node identityN = partNL.item(0);
				NodeList inner = ((Element) identityN).getElementsByTagName("node"); 	
				addGuid = Guid.parseGuid(getNodeText(inner.item(0)));
				inner = ((Element) identityN).getElementsByTagName("network");
				netGuid = Guid.parseGuid(getNodeText(inner.item(0)));
				NetIdentifier gateway = new NetIdentifier(addGuid, netGuid);
				
				partNL = ((Element) gatewayN).getElementsByTagName("interface");
				interfaceN = partNL.item(0);
				inner = ((Element) interfaceN).getElementsByTagName("node"); 	
				addGuid = Guid.parseGuid(getNodeText(inner.item(0)));
				inner = ((Element) interfaceN).getElementsByTagName("network");
				netGuid = Guid.parseGuid(getNodeText(inner.item(0)));
				NetIdentifier interfc = new NetIdentifier(addGuid, netGuid);
			
				router.setDefaultGateway(net, gateway, interfc);
			}
		}

    try {
      nl = doc.getElementsByTagName("interceptorInjector");
      n = nl.item(0);
      InterceptorInjector.getInterceptorInjector().setListenPort(
              Integer.parseInt( ((Element) n).getAttribute("listenPort"))
              );
    } catch (Exception e) {
      log.error(e);
    }
	}

	/**
	 * Return node with the name.
	 */
	private Node getNodeWithName(Node rodic, String jmeno) {
		NodeList nl = rodic.getChildNodes();
		Node n;
		for (int i = 0; i < nl.getLength(); i++) {
			n = nl.item(i);
			if (n.getNodeName().equals(jmeno) == true) {
				return n;
			}
		}
		return null;
	}

	/**
	 * Return text from node.
	 */
	private String getNodeText(Node n) {
		NodeList nl = n.getChildNodes();
		for (int j = 0; j < nl.getLength(); j++) {
			Node parser = nl.item(j);
			if (parser.getNodeValue().trim().length() > 0) {
				return parser.getNodeValue().trim();
			}
		}
		return null;
	}

	public void sendCapsule(TransmitCapsule c) {
		if(c.getOver() == null){
			c.setOver(router.getDirectionTo(c.getDest()));
		}
		if(c.getPrev() == null){
			c.setPrev(router.getLocalNetIdentInDirection(c.getOver()));
		}
		if(c.getSrc() == null){
			c.setSrc(router.getLocalNetIdentInDirection(c.getOver()));
		}
		
		outQueue.add(c);
	}
	
	public void injectCapsule(CodePkgIdent executedCode, Guid processIdent,
			NetIdentifier destination, TransmitData payload, boolean executeContinuously) throws DestinationNotAvailableException {
		
		TransmitCapsule tc = new TransmitCapsule();		
		tc.setDest(destination);		
		tc.setHash(executedCode);
		tc.setLocalApl(processIdent);
		tc.setData(payload);
		tc.setExecuteContinuously(executeContinuously);

		sendCapsule(tc);
		
	}
	
	public void injectCapsule(CodePkgIdent executedCode, Guid processIdent,
			NetIdentifier destination, TransmitData payload)
			throws DestinationNotAvailableException {
		
		injectCapsule(executedCode, processIdent, destination, payload, false);	
	}
	
	public String storeApplication(String localPath) throws AplStoreException {
		CodePkgIdent ident = codeStoreManager.storeLocalCodeFromFile(localPath);
		if (ident != null)
			return ident.toString();
		else
			return null;
	}

	/**
	 * Run application basely on name. The name can by package name of package
	 * identification in hexadecimal format.
	 * 
	 * @param name
	 *            name or identification in hexadecimal format of the
	 *            application
	 * @param attr
	 *            attributes
	 */
	public Guid runApplication(String name, String[] attr)
			throws ExecuteException {
		return runApplication(name, attr, null);
	}

	public Guid runApplication(CodePkgIdent appIdent, String[] attr)
			throws ExecuteException {
		return runApplication(appIdent, attr, null);
	}

	public Guid runApplication(String name, String[] attr, Guid parentAppl)	throws ExecuteException {
		
		try {
			// finding application package identification
			CodePkgIdent ident = null;
			CodePackage cp = repository.getCodePkg(name);
			
			if (cp != null) {				
				ident = cp.getPkgIdentification();
				
			} else {
				ident = CodePkgIdent.parse(name);
			}
			if (ident != null) {
				// run the application
				return scheduler.runApplication(ident, attr, null, parentAppl, cp);
				
			} else {
				log.error("Application: " + name + " was not found in repository");
				throw new ExecuteException("Application: " + name + " was not found in repository");
			}
			
		} catch (NumberFormatException e) {
			log.error("Application: " + name + " was not found in repository");
			throw new ExecuteException("Application: " + name + " was not found in repository");
			
		} catch (AplPkgException e) {
			log.error("Error during application " + name + "start", e); 
			throw new ExecuteException("Error during application " + name + "start");
		} 
	}

	public Guid runApplication(CodePkgIdent appIdent, String[] attr, Guid parentAppl) throws ExecuteException {
		
		CodePackage cp = null;
		
		try {
			cp = repository.getCodePkg(appIdent);
			
		} catch (AplPkgException e) {
			log.error("Application: " + appIdent + " was not found in repository", e); 
			throw new ExecuteException("Error during application " + appIdent + "start");
		} 
		
		if (cp != null) {
			log.error("Application: " + appIdent + " was not found in repository");
		}
		
		// run the application
		return scheduler.runApplication(appIdent, attr, null, parentAppl, cp);
	}

	public void waitForApplication(Guid app, Collection<Guid> applId)
			throws InterruptedException {
		scheduler.waitForApplication(app, applId);
	}

	public void shutDownServer() {
		System.exit(0);
	}

	static class FileAppenderWithoutLineEnds extends FileAppender {
		
		
		public FileAppenderWithoutLineEnds(PatternLayout pattern, String string, boolean b) throws IOException {
			super(pattern, string, b);
		}

		@Override
		public synchronized void doAppend(LoggingEvent event) {
			// TODO Auto-generated method stub
			String m = event.getMessage().toString().replace('\n', ' ');
			
			LoggingEvent l = new LoggingEvent(event.getFQNOfLoggerClass(), event.getLogger(), event.getTimeStamp(), 
					event.getLevel(), m, event.getThreadName(), event.getThrowableInformation(), 
					event.getNDC(), event.getLocationInformation(), event.getProperties());
			
			super.doAppend(l);
		}
	}

	/**
	 * Starts the server.
	 */
	public static void main(String[] args) throws Exception {
		
		if (args.length > 0) {
            /* use number X in settingsX.xml filename for identification */
            String configFileIdentifier = args[0].substring(args[0].indexOf('.') - 1, args[0].indexOf('.'));

			PatternLayout pattern = new PatternLayout();
			pattern.setConversionPattern("%d{dd.MM. HH:mm:ss,SSS};" + configFileIdentifier + "; %p; [%c]; %m%n");
			FileAppender fa = new FileAppenderWithoutLineEnds(pattern, "log/log" + configFileIdentifier + ".txt", false);
			fa.activateOptions();
			Logger logger = Logger.getRootLogger();
			logger.addAppender(fa);
			
			
			pattern = new PatternLayout();
			pattern.setConversionPattern("%d{dd.MM. HH:mm:ss,SSS};" + configFileIdentifier + "; %p; [%c]; %m%n");
			fa = new FileAppenderWithoutLineEnds(pattern, "log/logError"+ configFileIdentifier + ".txt", false);
			fa.activateOptions();
			fa.setThreshold(Level.ERROR);			
			logger.addAppender(fa);

			new Kernel(args[0]);
		} else {
			new Kernel();
		}
	}

	/**
	 * Getter of the property <tt>outQueue</tt>
	 * 
	 * @return Returns the outQueue.
	 * @uml.property name="outQueue"
	 */
	public OutgoingQueue getOutQueue() {
		return outQueue;
	}

	/**
	 * Getter of the property <tt>inQueue</tt>
	 * 
	 * @return Returns the inQueue.
	 * @uml.property name="inQueue"
	 */
	public IncomingQueue getInQueue() {
		return inQueue;
	}

	/**
	 * Getter of the property <tt>repository</tt>
	 * 
	 * @return Returns the repository.
	 * @uml.property name="repository"
	 */
	public IRepository getRepository() {
		return repository;
	}

	/**
	 * @uml.property name="scheduler"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="kernel:san.core.Scheduler"
	 */
	private Scheduler scheduler = null;

	/**
	 * Getter of the property <tt>scheduler</tt>
	 * 
	 * @return Returns the scheduler.
	 * @uml.property name="scheduler"
	 */
	public Scheduler getScheduler() {
		return scheduler;
	}

	/**
	 * Setter of the property <tt>scheduler</tt>
	 * 
	 * @param scheduler
	 *            The scheduler to set.
	 * @uml.property name="scheduler"
	 */
	public void setScheduler(Scheduler scheduler) {
		this.scheduler = scheduler;
	}

	/**
	 * @uml.property name="interpretManager"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="kernel:san.interpreter.InterpretManager"
	 */
	private InterpretManager interpretManager = null;

	/**
	 * Getter of the property <tt>interpretManager</tt>
	 * 
	 * @return Returns the interpretManager.
	 * @uml.property name="interpretManager"
	 */
	public InterpretManager getInterpretManager() {
		return interpretManager;
	}

	/**
	 * Setter of the property <tt>interpretManager</tt>
	 * 
	 * @param interpretManager
	 *            The interpretManager to set.
	 * @uml.property name="interpretManager"
	 */
	public void setInterpretManager(InterpretManager interpretManager) {
		this.interpretManager = interpretManager;
	}

	/**
	 * Send data from process to another process.
	 */
	public void sendDataToProcess(Guid from, Guid to, TransmitData data) {
		Process p = scheduler.getProcess(to);
		if (p != null && p instanceof ProcessApplication) {
			((ProcessApplication) p).fireOnSendDataListeners(from, data);
		} else {
      log.debug("No such application with guid " + to);
    }
	}

	/**
	 * Send data from process to another process.
	 */
	public void sendDataToProcess(Guid from, CodePkgIdent processCode,
			TransmitData data) {
		Collection<Process> process = scheduler.getProcess(processCode);
		for (Process p : process) {
			if (p != null && p instanceof ProcessApplication) {
				((ProcessApplication) p).fireOnSendDataListeners(from, data);
			}
		}
	}
	
	public IRoutingTableControl getRoutingModification() {
		return router.getRoutingTable();
	}

	/**
	 * @see san.core.IServerControl#createConsole()
	 */
	public void createConsole(Guid id) {
		// get process
		Process process = scheduler.getProcess(id);
		// create console
		IConsole console = null;
		if (textConsole) {
			try {
				console = new TextConsole();
			} catch (Exception e) {
				System.err.println("Opravdu to vyhodilo");
			}
		} else {
			WindowConsoleContainer wcc = new WindowConsoleContainer();
			console = wcc.getConsole();
			// show graphic console
			wcc.setVisible(true);
		}
		// set process the console
		process.setConsole(console);

	}

	/**
	 * Getter of the property <tt>router</tt>
	 * 
	 * @return Returns the router.
	 * @uml.property name="router"
	 */
	public Router getRouter() {
		return router;
	}

	/**
	 * Getter of the property <tt>netInterfaces</tt>
	 * 
	 * @return Returns the netInterfaces.
	 * @uml.property name="netInterfaces"
	 */
	public NetInterfaces getNetInterfaces() {
		return netInterfaces;
	}

	/**
	 * @return the softState
	 */
	public ISoftState getSoftState() {
		return softState;
	}

	public CodeStoreManager getCodeStoreManager() {
		return codeStoreManager;
	}	
	
	public boolean isMyInterfaceOrNull(NetIdentifier i) {
		return (i == null || netInterfaces.getNetInterfaces().containsKey(i));
	}

	public CapsuleTrafficLogger getCapsuleTrafficLogger() {
		return capsuleTrafficLogger;
	}

	@Override
	public void codeStored(CodePkgIdent codeIdent) {
		scheduler.codeExist(codeIdent);		
	}

	@Override
	public void codeStored(TransmitCapsule capsule) {
		scheduler.codeExist(capsule.getHash());		
	}

  public Guid getInitGuid() {
    return initGuid;
  }

  
}
