/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import san.applicationRepository.CodePackage;
import san.ident.Guid;
import san.interpreter.IInterpretingContext;
import san.console.IConsole;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Each process parent class.<br>
 * @author Michal Rejda
 * @since 20.10.2007
 * @version 1.0
 */
public class Process {

	/**
	 * <b>Title:</b> Smart Active Node<br>
	 * <b>Description:</b> Priority of the process.<br>
	 * @author Michal Rejda
	 * @since 26.10.2007
	 * @version 1.0
	 */
	public enum PriorityClass {
		LOW, NORMAL, HIGH, REAL_TIME
	}
	/**
	 * <b>Title:</b> Smart Active Node<br>
	 * <b>Description:</b> States of the process.<br>
	 * @author Michal Rejda
	 * @since 25.10.2007
	 * @version 1.0
	 */
	public enum State {
		RUNNING, READY_TO_RUN, STOPPED, ASLEEP, FINISHED
	}

	/**
	 * This constant is for advantage of priority when this process is running
	 * now. This constant is used by scheduler by process planing.
	 */
	protected static final long ADVANTAGE_FOR_RUNNING = 10;
	/**
	 * @uml.property name="state"
	 */
	protected State state = State.READY_TO_RUN;
	/**
	 * @uml.property name="priorityClass"
	 */
	protected PriorityClass priorityClass = PriorityClass.NORMAL;
	/**
	 * @uml.property name="codePackage"
	 * @uml.associationEnd multiplicity="(1 1)" aggregation="composite"
	 *                     inverse="executable:san.applicationRepository.CodePackage"
	 */
	protected CodePackage codePackage = null;
	/**
	 * @uml.property name="interpretingContext"
	 * @uml.associationEnd multiplicity="(1 1)" aggregation="composite"
	 *                     inverse="executable:san.interpreter.IInterpretingContext"
	 */
	protected IInterpretingContext executionContext = null;
	/**
	 * @uml.property name="kernel"
	 * @uml.associationEnd readOnly="true" inverse="process:san.core.Kernel"
	 */
	protected Kernel kernel;
	/**
	 * @uml.property name="id"
	 */
	protected Guid id = null;
	/**
	 * @uml.property name="parent"
	 */
	private Guid parent = null;
	/**
	 * @uml.property name="console"
	 */
	protected IConsole console = null;

	public Process(Kernel kernel) {
		this.kernel = kernel;
	}

	/**
	 * @uml.property name="id"
	 */
	public Guid getId() {
		return id;
	}

	/**
	 * Getter of the property <tt>codePackage</tt>
	 * @return Returns the codePackage.
	 * @uml.property name="codePackage"
	 */
	public CodePackage getCodePackage() {
		return codePackage;
	}

	/**
	 * Setter of the property <tt>codePackage</tt>
	 * @param codePackage The codePackage to set.
	 * @uml.property name="codePackage"
	 */
	public void setCodePackage(CodePackage codePackage) {
		this.codePackage = codePackage;
	}

	/**
	 * Getter of the property <tt>interpretingContext</tt>
	 * @return Returns the executionContext.
	 * @uml.property name="interpretingContext"
	 */
	public IInterpretingContext getInterpretingContext() {
		return executionContext;
	}

	/**
	 * Setter of the property <tt>interpretingContext</tt>
	 * @param interpretingContext The executionContext to set.
	 */
	public void setInterpretingContext(IInterpretingContext interpretingContext) {
		executionContext = interpretingContext;
	}

	/**
	 * Getter of the property <tt>kernel</tt>
	 * @return Returns the kernel.
	 * @uml.property name="kernel"
	 */
	Kernel getKernel() {
		return kernel;
	}

	/**
	 * Getter of the property <tt>state</tt>
	 * @return Returns the state.
	 * @uml.property name="state"
	 */
	public State getState() {
		return state;
	}

	/**
	 * Setter of the property <tt>state</tt>
	 * @param state The state to set.
	 * @uml.property name="state"
	 */
	public void setState(State state) {
		this.state = state;
	}

	/**
	 * @uml.property name="startInterpretingAt"
	 */
	private long startInterpretingAt;

	/**
	 * Getter of the property <tt>startInterpretingAt</tt>
	 * @return Returns the startInterpretingAt.
	 * @uml.property name="startInterpretingAt"
	 */
	public long getStartInterpretingAt() {
		return startInterpretingAt;
	}

	/**
	 * @uml.property name="endedInterpretingAt"
	 */
	private long endedInterpretingAt;

	/**
	 * Getter of the property <tt>endedInterpretingAt</tt>
	 * @return Returns the endedInterpretingAt.
	 * @uml.property name="endedInterpretingAt"
	 */
	public long getEndedInterpretingAt() {
		return endedInterpretingAt;
	}

	/**
	 * Setter of the property <tt>endedInterpretingAt</tt>
	 * @param endedInterpretingAt The endedInterpretingAt to set.
	 * @uml.property name="endedInterpretingAt"
	 */
	public void setEndedInterpretingAt(long endedInterpretingAt) {
		this.endedInterpretingAt = endedInterpretingAt;
	}

	/**
	 * Setter of the property <tt>startInterpretingAt</tt>
	 * @param startInterpretingAt The startInterpretingAt to set.
	 * @uml.property name="startInterpretingAt"
	 */
	public void setStartInterpretingAt(long startInterpretingAt) {
		this.startInterpretingAt = startInterpretingAt;
	}

	/**
	 * @uml.property name="lastUsedQuantum"
	 */
	private long lastUsedQuantum;

	/**
	 * Getter of the property <tt>lastUsedQuantum</tt>
	 * @return Returns the lastUsedQuantum.
	 * @uml.property name="lastUsedQuantum"
	 */
	public long getLastUsedQuantum() {
		return lastUsedQuantum;
	}

	/**
	 * Setter of the property <tt>lastUsedQuantum</tt>
	 * @param lastUsedQuantum The lastUsedQuantum to set.
	 * @uml.property name="lastUsedQuantum"
	 */
	public void setLastUsedQuantum(long lastUsedQuantum) {
		this.lastUsedQuantum = lastUsedQuantum;
	}

	/**
	 * @uml.property name="basePriority"
	 */
	private long basePriority;

	/**
	 * Getter of the property <tt>basePriority</tt>
	 * @return Returns the basePriority.
	 * @uml.property name="basePriority"
	 */
	public long getBasePriority() {
		return basePriority;
	}

	/**
	 * Setter of the property <tt>basePriority</tt>
	 * @param basePriority The basePriority to set.
	 * @uml.property name="basePriority"
	 */
	public void setBasePriority(long basePriority) {
		this.basePriority = basePriority;
	}

	/**
	 * @uml.property name="nicePriority"
	 */
	private long nicePriority;

	/**
	 * Getter of the property <tt>nicePriority</tt>
	 * @return Returns the nicePriority.
	 * @uml.property name="nicePriority"
	 */
	public long getNicePriority() {
		return nicePriority;
	}

	/**
	 * Setter of the property <tt>nicePriority</tt>
	 * @param nicePriority The nicePriority to set.
	 * @uml.property name="nicePriority"
	 */
	public void setNicePriority(long nicePriority) {
		this.nicePriority = nicePriority;
	}

	/**
	 * Return priority number. Bigger number is smaller priority. It is SUM of
	 * base priority, last used quantum, negative nice priority and negative
	 * ADVANTAGE_FOR_RUNNING constant if is running..
	 */
	public long getPriorityNumber() {
		if (state == State.RUNNING)
			return basePriority + lastUsedQuantum - nicePriority
					- ADVANTAGE_FOR_RUNNING;
		else
			return basePriority + lastUsedQuantum - nicePriority;
	}

	/**
	 * Setter of the property <tt>id</tt>
	 * @param id The id to set.
	 * @uml.property name="id"
	 */
	public void setId(Guid id) {
		this.id = id;
	}

	/**
	 * Getter of the property <tt>priorityClass</tt>
	 * @return Returns the priorityClass.
	 * @uml.property name="priorityClass"
	 */
	public PriorityClass getPriorityClass() {
		return priorityClass;
	}

	/**
	 * Setter of the property <tt>priorityClass</tt>
	 * @param priorityClass The priorityClass to set.
	 * @uml.property name="priorityClass"
	 */
	public void setPriorityClass(PriorityClass priorityClass) {
		this.priorityClass = priorityClass;
	}

	/**
	 * Getter of the property <tt>parent</tt>
	 * @return Returns the parent.
	 * @uml.property name="parent"
	 */
	public Guid getParent() {
		return parent;
	}

	/**
	 * Setter of the property <tt>parent</tt>
	 * @param parent The parent to set.
	 * @uml.property name="parent"
	 */
	public void setParent(Guid parent) {
		this.parent = parent;
	}

	/**
	 * @return the console
	 * @uml.property name="console"
	 */
	public IConsole getConsole() {
		return console;
	}

	/**
	 * @param console the console to set
	 * @uml.property name="console"
	 */
	public void setConsole(IConsole console) {
		this.console = console;
	}
}
