/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import san.codeDistribution.strategies.CodeDistributionStrategy;
import san.core.Process.PriorityClass;
import san.core.Process.State;
import san.data.TransmitCapsule;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.incomingQueue.IncomingQueue;
import san.interpreter.IInterpreter;
import san.interpreter.InterpretManager;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;

import san.applicationRepository.AplPkgException;
import san.applicationRepository.CodePackage;
import san.applicationRepository.IRepository;

import java.util.Collection;

import org.apache.log4j.Logger;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class select capsules from incoming queue and
 * schedule theirs interpreting. This class schedule applications interpreting
 * too.
 * @author Michal Rejda
 * @since 21.10.2007
 * @version 2.0
 */
public class Scheduler implements IScheduler {
	
	private static final transient Logger log = Logger.getLogger(Scheduler.class);

	/** Max number of running capsules. */
	public static final int CAPSULES_MAX_RUNNING = 4;
	/** Time between scheduling capsules. */
	public static final long CAPSULES_INTERVAL_SLEEP = 100;
	/** Time between scheduling applications. */
	public static final long APPLICATIONS_INTERVAL_SLEEP = 100;
	
	public static final long LOW_PRIORITY_QUANTUM = 10;
	public static final long NORMAL_PRIORITY_QUANTUM = 100;
	public static final long HIGT_PRIORITY_QUANTUM = 200;
	public static final long REAL_TIME_PRIORITY_QUANTUM = 500;
	
	private Thread capsulesPreparationThread;
	private Thread capsulesSchedulerThread;
	private Thread applicationsSchedulerThread;
	/** Lock for lock in waiting and executing collections of capsules. */
	private Object capsulesLock = new Object();
	/** Lock for lock in waiting and executing collections of applications. */
	private Object applicationsLock = new Object();
	private boolean running;
	
	private long appStartedCount = 0;
	private long appFinishedCount = 0;
	private long capsulesStartedCount = 0;
	private long capsulesFinishedCount = 0;
	/**
	 * @uml.property name="codeRepository"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="scheduler:san.applicationRepository.IRepository"
	 */
	private IRepository codeRepository = null;
	/**
	 * @uml.property name="kernel"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="scheduler:san.core.Kernel"
	 */
	private Kernel kernel = null;
	/**
	 * @uml.property name="interpretManager"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="scheduler:san.interpreter.InterpretManager"
	 */
	private InterpretManager interpretManager = null;
	/**
	 * @uml.property name="incomingQueue"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="scheduler:san.incomingQueue.IncomingQueue"
	 */
	private IncomingQueue incomingQueue = null;
	/**
	 * @uml.property name="waitingForCode"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="scheduler:san.data.TransmitCapsule"
	 */
	private Collection<TransmitCapsule> capsulesWaitingForCode;
	/**
	 * @uml.property name="haveCode"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="scheduler:san.data.TransmitCapsule"
	 */
	private Queue<TransmitCapsule> capsulesWeHaveCodeFor;
	/**
	 * @uml.property name="runningApplications"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="scheduler:san.core.Process"
	 */
	private Queue<Process> runningApplications;
	/**
	 * @uml.property name="runningCapsules"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="scheduler:san.core.Process"
	 */
	private Queue<Process> runningCapsules;
	/**
	 * @uml.property name="waitingApplications"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="scheduler:san.core.Process"
	 */
	private Queue<Process> waitingApplications;
	/**
	 * @uml.property name="waitingCapsules"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="scheduler:san.core.Process"
	 */
	private Queue<Process> waitingCapsules;

	private CodeDistributionStrategy codeDistributionStrategy;

	public Scheduler(Kernel kernel, IRepository repository,
			IncomingQueue inQueue, InterpretManager interpretMgr, CodeDistributionStrategy codeDistributionStrategy) {
		
		this.kernel = kernel;
		this.codeRepository = repository;
		this.incomingQueue = inQueue;
		this.interpretManager = interpretMgr;
		this.codeDistributionStrategy = codeDistributionStrategy;
		
		this.capsulesWaitingForCode = new LinkedList<TransmitCapsule>();
		this.capsulesWeHaveCodeFor = new LinkedList<TransmitCapsule>();
		this.runningApplications = new LinkedList<Process>();
		this.waitingApplications = new LinkedList<Process>();
		this.runningCapsules = new LinkedList<Process>();
		this.waitingCapsules = new LinkedList<Process>();
		this.running = true;
		
		this.applicationsSchedulerThread = new Thread() {
			@Override
			public void run() {
				while (running) {
					try {
						Thread.sleep(APPLICATIONS_INTERVAL_SLEEP);
					}
					catch (InterruptedException e) {/* not a exception */
					}
					scheduleApplication();
				}
			}
		};
		
		this.capsulesSchedulerThread = new Thread() {

			@Override
			public void run() {
				while (running) {
					try {
						Thread.sleep(CAPSULES_INTERVAL_SLEEP);
					}
					catch (InterruptedException e) {/* not a exception */
					
					}
					scheduleCapsule();
				}
			}
		};
		
		this.capsulesPreparationThread = new Thread() {

			@Override
			public void run() {
				while(running)
					prepareCapsuleForExecuting();
			}
		};
		
		this.applicationsSchedulerThread.start();
		this.capsulesSchedulerThread.start();
		this.capsulesPreparationThread.start();
	}

	/**
	 * If is count of running and waiting capsules less then
	 * CAPSULES_MAX_RUNNING, prepare new capsule to executing (give it into
	 * waiting queue). If is count of waiting and executing capsules more then
	 * CAPSULES_MAX_RUNNING it wait for end of executing some code. If isn't
	 * capsule for preparing for executing do nothing.
	 */
	private void prepareCapsuleForExecuting() {
		
		TransmitCapsule capsule = null;		
		CodePackage cp = null;		
		
		synchronized (capsulesWeHaveCodeFor) {	//kapsule pro ktere mame urcite kod
			if (capsulesWeHaveCodeFor.size() > 0){
				capsule = capsulesWeHaveCodeFor.poll();
			}
		}
		
		if (capsule == null) {

			capsule = incomingQueue.getCapsule();	//nove prichozi kapsule	
			
			if (capsule != null) {
				// ITS OK
			}
			else{
				log.debug("capsule je null!");	//nastava po interrupt()
				return;
			}
		}
		
		try {
			cp = codeRepository.getCodePkg(capsule.getHash());
		}
		catch (AplPkgException e) {
			log.error(e.getMessage(), e);
		}
		
		
		if ( cp != null ) {		//mame kod pro kapsuli			
			
			if (log.isDebugEnabled()) {
				log.debug("code for " + capsule + " exists in rep");
			}
	
			// preparing capsule for executing
			ProcessCapsule e = new ProcessCapsule(kernel);
			/* FIXME - here is capsule taken out from incomming queue and prepared for executing
			 * here should be set priority of the capsule process
			 */ 
			e.setCapsule(capsule);
			e.setCodePackage(cp);
			synchronized (capsulesLock) {
				while(true){
					try {
						while (runningCapsules.size()+ waitingCapsules.size() > CAPSULES_MAX_RUNNING) {
							capsulesLock.wait();
						}
						waitingCapsules.add(e);
						capsulesLock.notifyAll();
						break;
					}
					catch (InterruptedException ex) {
						log.error(ex);
					}
				}
			}
			
			
		} else {	//nemame kod pro kapsuli
			
			if (log.isDebugEnabled()) {
				log.debug("code for " + capsule + " doesnt exists in rep");
			}
			
			synchronized (capsulesWaitingForCode) {
				capsulesWaitingForCode.add(capsule);
			}

			kernel.getRepository().tryToSendNextRequestForApp(capsule);
		}
	}

	/**
	 * Remove first waiting capsule from queue and give it for interpreting. If
	 * another capsule is interpreted, terminate it and give it into waiting
	 * queue.
	 */
	private void scheduleCapsule() {
		synchronized (capsulesLock) {
			try {
				// waiting for capsule
				while (waitingCapsules.size() <= 0)
					capsulesLock.wait();
				// taking out capsule
				Process process = waitingCapsules.poll();
				// getting interpreter
				IInterpreter interpreter = interpretManager.getInterpreter(process.getCodePackage());
				// terminating executing
				Process terminate = interpreter.terminate();
				if (terminate != null) {
					// add terminate capsule into waiting queue
					waitingCapsules.add(terminate);
				}
				// start running scheduled capsule
				runningCapsules.add(process);
				capsulesStartedCount++;
				if (log.isDebugEnabled()) {
					log.debug("capsule added in running&waiting: " + process.getCodePackage().getPkgName() + ", size r=" + runningCapsules.size() + ", w=" + waitingCapsules.size() +  ", started sum=" + capsulesStartedCount);
				}
			
				
				interpreter.runProcess(process);
			}
			catch (InterruptedException e) {
				log.error(e);
			}
		}
	}

	/**
	 * Remove first waiting application from queue and give it for interpreting.
	 * If another application is interpreted, terminate it and give it into
	 * waiting queue.
	 */
	private void scheduleApplication() {
		synchronized (applicationsLock) {
			try {
				// waiting for some application
				while (waitingApplications.size() <= 0)
					applicationsLock.wait();
				Process choose = null;
				Process process = null;
				boolean inWaiting = false;
				long now = System.currentTimeMillis();
				for (Iterator<Process> iter = runningApplications.iterator(); iter.hasNext();) {
					 process = (Process) iter.next();
					//remove finished process
					if(process.getState() == State.FINISHED) {
						iter.remove();
						continue;
					}
					//set last used quantum
					process.setLastUsedQuantum(now - process.getStartInterpretingAt());
					//select process with minimum number of priority
					if(choose == null) {
						choose = process;
					} else {
						if(choose.getPriorityNumber() > process.getPriorityNumber()){
							choose = process;
						}
					}
					
				}
				for (Iterator<Process> iter = waitingApplications.iterator(); iter.hasNext();) {
					 process = (Process) iter.next();
					//set process last used quantum to half
					process.setLastUsedQuantum(process.getLastUsedQuantum() >>> 1);
					//select process with minimum number of priority
					if(choose == null) {
						choose = process;
						inWaiting = true;
					} else {
						if(choose.getPriorityNumber() > process.getPriorityNumber()){
							switch(process.getState()){
								case STOPPED:
								case READY_TO_RUN:
									choose = process;
									inWaiting = true;
									break;
							}
						}
					}
				}
				if(choose != null && inWaiting) {
					waitingApplications.remove(choose);
					// getting interpreter
					IInterpreter interpreter = interpretManager.getInterpreter(choose.getCodePackage());
					// terminating executing
					Process terminate = interpreter.terminate();
					if (terminate != null) {
						terminate.setState(State.STOPPED);
						// add terminate application into waiting queue
						waitingApplications.add(terminate);
					}
					// start running scheduled application
					choose.setState(State.READY_TO_RUN);
					choose.setStartInterpretingAt(System.currentTimeMillis());
					runningApplications.add(choose);
					
					appStartedCount++;
					
					if (log.isDebugEnabled()) {
						log.debug("app added in running&waiting: " + choose.getCodePackage().getPkgName() + ", size r=" + runningApplications.size() + ", w=" + waitingApplications.size() +  ", started sum=" + appStartedCount);
					}
					
					interpreter.runProcess(choose);
				}
			}
			catch (InterruptedException e) {
				log.error(e);
			}
		}
	}

	public Guid runApplication(CodePkgIdent appIdent, String[] attr, PriorityClass priority, Guid parent, CodePackage code){
		
		/* FIXME -  here want user to run application
		 * I do not know if Security manager should do something like setting priority of the process 
		 */
		synchronized (applicationsLock) {
			
			ProcessApplication ea = new ProcessApplication(kernel);
			if(priority == null)
				priority = PriorityClass.NORMAL;
			ea.setPriorityClass(priority);
			switch(priority){
				case LOW:
					ea.setBasePriority(LOW_PRIORITY_QUANTUM);
					break;
				case HIGH:
					ea.setBasePriority(HIGT_PRIORITY_QUANTUM);
					break;
				case REAL_TIME:
					ea.setBasePriority(REAL_TIME_PRIORITY_QUANTUM);
					break;
				case NORMAL:
				default:
					ea.setBasePriority(NORMAL_PRIORITY_QUANTUM);
					break;
			}
			ea.setId(Guid.randomGuid());	//create new application id
			ea.setArgs(attr);
			ea.setCodePackage(code);
			ea.setState(State.READY_TO_RUN);
			ea.setParent(parent);
			
			//set console
			if(parent != null) {
				Process parentProcess = getProcess(parent);
				if(parentProcess != null) {
					ea.setConsole(parentProcess.getConsole());
				}
			}
			waitingApplications.add(ea);
			applicationsLock.notifyAll();
			return ea.getId();
			
			
		}
	}
	
	public Guid runApplication(CodePkgIdent appIdent, String[] attr, PriorityClass priority, Guid parent)
			throws ExecuteException {
		
		try {
			CodePackage code = codeRepository.getCodePkg(appIdent);
			return runApplication(appIdent, attr, priority, parent, code);
		}catch (AplPkgException e) {
			throw new ExecuteException(e.getMessage());
		}
	}

	public void waitForApplication(Guid app, Collection<Guid> applId) throws InterruptedException {
		synchronized (applicationsLock) {
			Process proc = null;
			boolean waitingEnded = true;
			do{
				waitingEnded = true;
				for (Guid g : applId) {
					proc = getProcess(g);
					if(proc != null) {
						waitingEnded = false;
						break;
					}
				}
				if(waitingEnded == false)
					applicationsLock.wait();
			} while(waitingEnded == false);
		}
	}
	
	public void intepretingEnded(Process code) {
				
		if (code instanceof ProcessCapsule) {
			synchronized (capsulesLock) {			
				
				runningCapsules.remove(code);
				waitingCapsules.remove(code);
				
				capsulesFinishedCount++;
				
				if (log.isDebugEnabled()) {
					log.debug("capsule removed from running&waiting: " + code.getCodePackage().getPkgName() + ", size r=" + runningCapsules.size() + ", size w=" + waitingCapsules.size()+ ", finished sum=" + capsulesFinishedCount);
				}
				
				capsulesLock.notifyAll();
			}
		} else if (code instanceof ProcessApplication) {
			synchronized (applicationsLock) {
				
				runningApplications.remove(code);
				waitingApplications.remove(code);
				
				appFinishedCount++;
				
				if (log.isDebugEnabled()) {
					log.debug("app removed from running: " + code.getCodePackage().getPkgName() + " , size r=" + runningApplications.size() + ", size w=" + waitingApplications.size() + ", finished sum=" + appFinishedCount);				
				}
		
				applicationsLock.notifyAll();
			}
		}
		
		kernel.getRepository().dontProtectAppFromDeleteAnyLonger(code.codePackage.getPkgIdentification());		
		kernel.getRepository().tryToSendNextRequestForApp();
	}

	public Process getProcess(Guid ident) {
		synchronized (applicationsLock) {
			for (Process p : runningApplications) {
				if(p.getId().equals(ident))
					return p;
			}
			for (Process p : waitingApplications) {
				if(p.getId().equals(ident))
					return p;
			}
		}
		return null;
	}
	
	/**
	 * Return collection of process that are using specific Code Package.
	 * @param processCode Code  Package used for process interpretation 
	 * @return collection of process
	 */
	public Collection<Process> getProcess(CodePkgIdent processCode) {
		Collection<Process> matchingProcess = new LinkedList<Process>();
		synchronized (applicationsLock) {
			for (Process p : runningApplications) {
				if(p.getCodePackage().getPkgIdentification().equals(processCode))
					matchingProcess.add(p);
			}
			for (Process p : waitingApplications) {
				if(p.getCodePackage().getPkgIdentification().equals(processCode))
					matchingProcess.add(p);
			}
		}
		return matchingProcess;
	}
	
	/**
	 * slouzi k informovani Scheduleru, ze zrovna prisel kod na kterej ceka 
	 */
	public void codeExist(CodePkgIdent codeIdent) {
		Collection<TransmitCapsule>	capsulesWeJustGotCodeFor = new LinkedList<TransmitCapsule>();
		synchronized (capsulesWaitingForCode) {
			TransmitCapsule tc;
			for (Iterator<TransmitCapsule> iterator = capsulesWaitingForCode.iterator(); iterator.hasNext();) {
				tc = (TransmitCapsule) iterator.next();
				if(tc.getHash().equals(codeIdent)){
					iterator.remove();
					capsulesWeJustGotCodeFor.add(tc);
					if (log.isDebugEnabled()) {
						log.debug("got code for " + tc);
					}
				}
			}
		}
		boolean changed = false;
		synchronized (this.capsulesWeHaveCodeFor) {
			changed = this.capsulesWeHaveCodeFor.addAll(capsulesWeJustGotCodeFor);
		}
		if(changed && capsulesPreparationThread.getState() != Thread.State.RUNNABLE) {
			capsulesPreparationThread.interrupt();
		}
	}

	/**
	 * Getter of the property <tt>interpretManager</tt>
	 * @return Returns the interpretManager.
	 * @uml.property name="interpretManager"
	 */
	public InterpretManager getInterpretManager() {
		return interpretManager;
	}

	/**
	 * Setter of the property <tt>interpretManager</tt>
	 * @param interpretManager The interpretManager to set.
	 * @uml.property name="interpretManager"
	 */
	public void setInterpretManager(InterpretManager interpretManager) {
		this.interpretManager = interpretManager;
	}

	/**
	 * Getter of the property <tt>incomingQueue</tt>
	 * @return Returns the incomingQueue.
	 * @uml.property name="incomingQueue"
	 */
	public IncomingQueue getIncomingQueue() {
		return incomingQueue;
	}

	/**
	 * Setter of the property <tt>incomingQueue</tt>
	 * @param incomingQueue The incomingQueue to set.
	 * @uml.property name="incomingQueue"
	 */
	public void setIncomingQueue(IncomingQueue incomingQueue) {
		this.incomingQueue = incomingQueue;
	}

	/**
	 * Getter of the property <tt>runningCapsules</tt>
	 * @return Returns the runningCapsules.
	 * @uml.property name="runningCapsules"
	 */
	public Queue<Process> getRunningCapsules() {
		return runningCapsules;
	}

	/**
	 * Getter of the property <tt>runningApplications</tt>
	 * @return Returns the runningApplication.
	 * @uml.property name="runningApplications"
	 */
	public Queue<Process> getRunningApplications() {
		return runningApplications;
	}

	/**
	 * Getter of the property <tt>waitingApplications</tt>
	 * @return Returns the waitingApplications.
	 * @uml.property name="waitingApplications"
	 */
	public Queue<Process> getWaitingApplications() {
		return waitingApplications;
	}

	/**
	 * Getter of the property <tt>waitingCapsules</tt>
	 * @return Returns the waitingCapsules.
	 * @uml.property name="waitingCapsules"
	 */
	public Queue<Process> getWaitingCapsules() {
		return waitingCapsules;
	}

	/**
	 * Getter of the property <tt>kernel</tt>
	 * @return Returns the kernel.
	 * @uml.property name="kernel"
	 */
	public Kernel getKernel() {
		return kernel;
	}

	/**
	 * Setter of the property <tt>kernel</tt>
	 * @param kernel The kernel to set.
	 * @uml.property name="kernel"
	 */
	public void setKernel(Kernel kernel) {
		this.kernel = kernel;
	}

	/**
	 * Setter of the property <tt>runningCapsules</tt>
	 * @param runningCapsules The runningCapsules to set.
	 * @uml.property name="runningCapsules"
	 */
	public void setRunningCapsules(Queue<Process> runningCapsules) {
		this.runningCapsules = runningCapsules;
	}

	/**
	 * Setter of the property <tt>waitingApplications</tt>
	 * @param waitingApplications The waitingApplications to set.
	 * @uml.property name="waitingApplications"
	 */
	public void setWaitingApplications(Queue<Process> waitingApplications) {
		this.waitingApplications = waitingApplications;
	}

	/**
	 * Setter of the property <tt>runningApplications</tt>
	 * @param runningApplications The runningApplication to set.
	 * @uml.property name="runningApplications"
	 */
	public void setRunningApplications(Queue<Process> runningApplications) {
		this.runningApplications = runningApplications;
	}

	/**
	 * Setter of the property <tt>waitingCapsules</tt>
	 * @param waitingCapsules The waitingCapsules to set.
	 * @uml.property name="waitingCapsules"
	 */
	public void setWaitingCapsules(Queue<Process> waitingCapsules) {
		this.waitingCapsules = waitingCapsules;
	}
}
