/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.data;

import java.io.Serializable;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Capsule for data transporting.<br>
 * @author Jan Boháč, Michal Rejda
 * @since 30.10.2006
 * @version 1.0
 */
public class TransmitCapsule implements Serializable {

	public static final long serialVersionUID = 1389978312364545421L;
	/**
	 * capsule identification
	 * @uml.property name="capsuleIdent"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.Guid"
	 */
	private Guid capsuleIdent = null;
	/**
	 * @uml.property name="src"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.NetIdentifier"
	 */
	private NetIdentifier src = null;
	/**
	 * @uml.property name="dest"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.NetIdentifier"
	 */
	private NetIdentifier dest = null;
	/**
	 * @uml.property name="over"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.NetIdentifier"
	 */
	private NetIdentifier over = null;
	
	/**
	 * @uml.property name="prev"
	 * @uml.associationEnd inverse="transmitCapsule:san.ident.NetIdentifier"
	 */
	private NetIdentifier prev = null;

	/**
	 * identification of application code that will be run when capsule income
	 * onto the server
	 * @uml.property name="hash"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.AplPkgIdent"
	 */
	private CodePkgIdent hash = null;
	/**
	 * identification of local application that use the capsule
	 * @uml.property name="localApl"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.Guid"
	 */
	private Guid localApl = null;
	/**
	 * Identification of application that created the capsule.
	 * @uml.property name="remoteApl"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.ident.Guid"
	 */
	private Guid remoteApl = null;
	/**
	 * @uml.property name="data"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="transmitCapsule:san.data.TransmitData"
	 */
	private TransmitData data = null;
	private boolean aplication; // indicates if are data in capsule transported
	private boolean needAplication;// indicates if the server need application
									// for the hash
	
	private boolean executeContinuously = true;

	public TransmitCapsule() {
		this.capsuleIdent = Guid.randomGuid();
		this.src = null;
		this.dest = null;
		this.over = null;
		this.hash = null;
		this.data = null;
		this.aplication = false;
		this.needAplication = false;
		this.executeContinuously = true;
	}

	public TransmitCapsule(TransmitCapsule c) {
		this.capsuleIdent = c.capsuleIdent;
		this.src = new NetIdentifier(c.src);
		this.dest = new NetIdentifier(c.dest);
		this.over = new NetIdentifier(c.over);
		this.prev = new NetIdentifier(c.prev);
		this.hash = c.hash;
		this.data = new TransmitData();
		this.data.setData(c.data.getData());
		this.aplication = c.aplication;
		this.needAplication = c.needAplication;
		this.localApl = c.localApl;
		this.remoteApl = c.remoteApl;
		this.executeContinuously = c.isExecuteContinuously();
	}

	/**
	 * Getter of the property <tt>data</tt>
	 * @return Returns the data.
	 * @uml.property name="data"
	 */
	public TransmitData getData() {
		return data;
	}

	/**
	 * Setter of the property <tt>data</tt>
	 * @param data The data to set.
	 * @uml.property name="data"
	 */
	public void setData(TransmitData data) {
		this.data = data;
	}

	/**
	 * Getter of the property <tt>hash</tt>
	 * @return Returns the hash.
	 * @uml.property name="hash"
	 */
	public CodePkgIdent getHash() {
		return hash;
	}

	/**
	 * Setter of the property <tt>hash</tt>
	 * @param hash The hash to set.
	 * @uml.property name="hash"
	 */
	public void setHash(CodePkgIdent hash) {
		this.hash = hash;
	}

	/**
	 * Getter of the property <tt>over</tt>
	 * @return Returns the over.
	 * @uml.property name="over"
	 */
	public NetIdentifier getOver() {
		return over;
	}

	/**
	 * Setter of the property <tt>over</tt>
	 * @param over The over to set.
	 * @uml.property name="over"
	 */
	public void setOver(NetIdentifier over) {
		this.over = over;
	}

	/**
	 * Getter of the property <tt>src</tt>
	 * @return Returns the src.
	 * @uml.property name="src"
	 */
	public NetIdentifier getSrc() {
		return src;
	}

	/**
	 * Setter of the property <tt>src</tt>
	 * @param src The src to set.
	 * @uml.property name="src"
	 */
	public void setSrc(NetIdentifier src) {
		this.src = src;
	}

	public String toString() {
		String text = "\nSrc: [" + src + "]\n"  
			+ "Dest:[" + dest + "]\n" 
			+ "Over:[" + over + "]\n"
			+ "Prev:[" + prev + "]\n"
			+ "Hash:[" + hash + "]\n"; 
			
			if(isAplication()){
				text += "[APPLICATION]\n";
			}
			if(isRequestForApplication()){
				text += "[REQ FOR APP]\n";
			}
						
			//+ "TransmitData:\n" + data + "\n";		
		return text;
	}

	/**
	 * @return the aplication
	 */
	public boolean isAplication() {
		return aplication;
	}

	/**
	 * @param aplication the aplication to set
	 */
	public void setAplication(boolean aplication) {
		this.aplication = aplication;
	}

	/**
	 * @return the needAplication
	 */
	public boolean isRequestForApplication() {
		return needAplication;
	}

	/**
	 * @param needAplication the needAplication to set
	 */
	public void setIsRequestForAplication(boolean needAplication) {
		this.needAplication = needAplication;
	}

	/**
	 * Getter of the property <tt>localApl</tt>
	 * @return Returns the localApl.
	 * @uml.property name="localApl"
	 */
	public Guid getLocalApl() {
		return localApl;
	}

	/**
	 * Setter of the property <tt>localApl</tt>
	 * @param localApl The localApl to set.
	 * @uml.property name="localApl"
	 */
	public void setLocalApl(Guid localApl) {
		this.localApl = localApl;
	}

	/**
	 * Getter of the property <tt>remoteApl</tt>
	 * @return Returns the remoteApl.
	 * @uml.property name="remoteApl"
	 */
	public Guid getRemoteApl() {
		return remoteApl;
	}

	/**
	 * Setter of the property <tt>remoteApl</tt>
	 * @param remoteApl The remoteApl to set.
	 * @uml.property name="remoteApl"
	 */
	public void setRemoteApl(Guid remoteApl) {
		this.remoteApl = remoteApl;
	}

	/**
	 * Getter of the property <tt>capsuleIdent</tt>
	 * @return Returns the capsuleIdent.
	 * @uml.property name="capsuleIdent"
	 */
	public Guid getCapsuleIdent() {
		return capsuleIdent;
	}

	/**
	 * Setter of the property <tt>capsuleIdent</tt>
	 * @param capsuleIdent The capsuleIdent to set.
	 * @uml.property name="capsuleIdent"
	 */
	public void setCapsuleIdent(Guid capsuleIdent) {
		this.capsuleIdent = capsuleIdent;
	}

	/**
	 * Getter of the property <tt>dest</tt>
	 * @return Returns the dest.
	 * @uml.property name="dest"
	 */
	public NetIdentifier getDest() {
		return dest;
	}

	/**
	 * Setter of the property <tt>dest</tt>
	 * @param dest The dest to set.
	 * @uml.property name="dest"
	 */
	public void setDest(NetIdentifier dest) {
		this.dest = dest;
	}

	/**
	 * @uml.property name="hopeLimit"
	 */
	private int hopeLimit;

	/**
	 * Getter of the property <tt>hopeLimit</tt>
	 * @return Returns the hopeLimit.
	 * @uml.property name="hopeLimit"
	 */
	public int getHopeLimit() {
		return hopeLimit;
	}

	/**
	 * Setter of the property <tt>hopeLimit</tt>
	 * @param hopeLimit The hopeLimit to set.
	 * @uml.property name="hopeLimit"
	 */
	public void setHopeLimit(int hopeLimit) {
		this.hopeLimit = hopeLimit;
	}

	
	/**
	 * Getter of the property <tt>prev</tt>
	 * @return Returns the prev.
	 * @uml.property name="prev"
	 */
	public NetIdentifier getPrev() {
		return prev;
	}

	/**
	 * Setter of the property <tt>prev</tt>
	 * @param prev The prev to set.
	 * @uml.property name="prev"
	 */
	public void setPrev(NetIdentifier prev) {
		this.prev = prev;
	}

	public boolean isExecuteContinuously() {
		return executeContinuously;
	}

	public void setExecuteContinuously(boolean executeContinuously) {
		this.executeContinuously = executeContinuously;
	}
}
