/*************************************************************************
 *                           Smart Active Node                           *
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.ii;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashSet;
import san.applicationRepository.SanCodePkg;
import san.core.ExecuteException;
import san.core.Kernel;
import san.core.Process;
import san.core.ProcessApplication;
import san.data.TransmitData;
import san.ident.Guid;
import san.interpreter.sanint.classLoading.ClassManager;
import san.interpreter.sanint.classLoading.Reference;

/**
 * This class is responsible for communication with ii in C. Interceptor in C
 * gathers IP packets from NF QUEUES and sends them through socket to this JAVA
 * ii. Java ii runs proper application and provides the payload of the packet to
 * it. On the other end, the application sends its payload to ii and this ii
 * sends it back to C injector. C injector injects baked packet into network.
 * 
 * @author kubajz
 */
public class InterceptorInjector extends Thread {

  /**
   * Apache log4j logger for this class
   * we cannot use it since SAN Localization is not capable of handling property files...
   */
  //private static final transient Logger log = Logger.getLogger(InterceptorInjector.class);
  /**
   * Guid of ii application
   */
  Guid myGuid;
  /**
   * reference to this ensures that only one ii can exist at time
   */
  private static InterceptorInjector singleton = null;
  /**
   * reference to Kernel
   */
  private Kernel kernel;
  /**
   * all created threads go there for sending packets
   */
  private HashSet<InjectorThread> threads = new HashSet<InjectorThread>();
  /**
   * port to listen on for incoming connections from ii clients
   */
  private int listenPort;

  /**
   * private constructor ensures that reference to this can be obtained only by static getter
   */
  private InterceptorInjector(Kernel kernel) {
    myGuid = Guid.randomGuid();
    this.kernel = kernel;
  }

  /**
   * This method gets called by active application that wants to send
   * payload back to legacy network
   * 
   * @param payload payload to be sent back to network
   */
  synchronized public void sendPayload(Packet payload) {
    //System.out.println("sendPayload...");
    for (InjectorThread injectorThread : threads) {
      injectorThread.setPayload(payload);
      injectorThread.run();
    }
  }

  /**
   * Stores payload into payload storage for gather by application identified
   * by guid parameter and runs appropriate application.
   * It is obvious if we don't know application's guid, we cannot pass it payload
   * and if application is run earlier, it 
   * 
   * @param guid guid of retreiver application
   * @param payload payload to send to application
   */
  synchronized public void storePayload(Packet payload) throws InterruptedException {
    Guid guid;

    try {
      //run application with Init as its parent - to innherit console etc.
      guid = kernel.runApplication(payload.getApplicationName(),
                                   payload.getApplicationParameters(),
                                   kernel.getInitGuid());

      //get process of application run
      Process p = kernel.getScheduler().getProcess(guid);

      if (p != null && p instanceof ProcessApplication) {
        ProcessApplication pa = (ProcessApplication) p;
        Reference thisRef = kernel.getInterpretManager().getRunningApplication(guid);
        while (thisRef == null) { //this can take a while - wait. Who waits, gets :]
          thisRef = kernel.getInterpretManager().getRunningApplication(guid);
          Thread.sleep(1L);
        }

        //get application's ClassManager
        ClassManager cman = new ClassManager((SanCodePkg) pa.getCodePackage());
        try {
          // fire application's receiveData method
          cman.runMethod(thisRef.getClassInfo().getName(),
                          "receiveData",
                          "(Lsan/ident/Guid;Lsan/data/TransmitData;)V",
                          new Object[]{kernel.getInitGuid(),
                          new TransmitData(payload.getPayload())},
                          thisRef,
                          false);
        } catch (Exception ex) {
          System.out.println("Cannot execute methods on application" +
                  payload.getApplicationName() + " with state " +
                  p.getState() + "\n" + ex);
          ex.printStackTrace();
        }
      }

    } catch (ExecuteException ex) {
      System.out.println("Cannot run application " + payload.getApplicationName());
    }



  }

  /**
   * Singleton pattern getter
   * @return reference to the only ii
   */
  public static InterceptorInjector getInterceptorInjector(Kernel kernel) {
    if (singleton == null) {
      singleton = new InterceptorInjector(kernel);
    }
    return singleton;
  }

  /**
   * If ii was created, you can use this method for referencing it.
   * @return reference to ii or null if serverControl was not set yet
   */
  public static InterceptorInjector getInterceptorInjector() throws Exception {
    if (singleton != null) {
      return singleton;
    } else {
      //throw new Exception(Localization.getLocalization().getString("M000056"));
      throw new Exception();
    }
  }

  @Override
  public void run() {
    System.out.println("SAN Interceptor Injector starting on port " + listenPort);
    ServerSocket serverSocket = null;
    boolean listening = true;

    try {
      serverSocket = new ServerSocket(listenPort);
    } catch (IOException e) {
      //log.error(Localization.getLocalization().getString("M000057", new Object[]{listenPort}));
      System.out.println("Cannot listen on port " + listenPort);
      return;
    }

    try {
      while (listening) {
        Socket clientSocket = serverSocket.accept();
        InterceptorThread interceptorThread = new InterceptorThread(clientSocket);
        InjectorThread injectorThread = new InjectorThread(clientSocket);
        threads.add(injectorThread);
        interceptorThread.start();
      }
      serverSocket.close();
    } catch (IOException ioe) {
      //log.error(ioe);
      System.out.println(ioe);
    }


  }

  public void setListenPort(int listenPort) {
    this.listenPort = listenPort;
  }

  public int getListenPort() {
    return listenPort;
  }

  /**
   *
   * @return guid of ii
   */
  public Guid getGuid() {
    return myGuid;
  }
}
