/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package san.ii;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.log4j.Logger;
import san.localization.Localization;

/**
 * Serializes a packet to stream. An ii packet looks like this:
 *
 * |AppName|Params|Payload|
 *
 * 2 Bytes AppNameLength
 * AppNameLength Bytes AppName
 * 2 Bytes AppParamCount
 * AppParamCount Strings
 * 2 Bytes PayloadLength
 * PayloadLength Bytes of Payload
 *
 * where String is:
 * 2 Bytes StringLength
 * StringLength Bytes Characters
 * @author kubajz
 */
public class PacketSerializer {

  private static final transient Logger log = Logger.getLogger(PacketSerializer.class);
  DataInputStream in;
  DataOutputStream out;

  public PacketSerializer(DataInputStream in) {
    this.in = in;
  }

  public PacketSerializer(DataOutputStream out) {
    this.out = out;
  }

  public PacketSerializer(DataInputStream in, DataOutputStream out) {
    this.in = in;
    this.out = out;
  }

  /**
   * Writes packet to data output stream. This assumes creating serializer
   * with data output stream or an IOexception is thrown
   * @param packet packet to be serialized and written
   */
  public void writePacket(Packet packet) throws IOException {
    out.writeShort(packet.getApplicationName().length());
    out.writeBytes(packet.getApplicationName());
    out.writeShort(packet.getApplicationParameters().length);

    //write parameters
    for (String param : packet.getApplicationParameters()) {
      out.writeShort(param.length());
      out.writeBytes(param);
    }

    out.writeShort(packet.getPayload().length);
    out.write(packet.getPayload(), 0, packet.getPayload().length);
    out.flush();
  }

  /**
   * Reads IP packet containing:
   * 2 Bytes AppNameLength
   * AppNameLength Bytes AppName
   * 2 Bytes AppParamCount
   * AppParamCount Strings
   * 2 Bytes PayloadLength
   * PayloadLength Bytes of Payload
   *
   * where String is:
   * 2 Bytes StringLength
   * StringLength Bytes Characters
   *
   * @return baked Packet
   */
  public Packet readPacket() {
    try {
      int appNameLength;
      String appName = null;
      int appParamCount;
      String[] appParams = null;
      int payloadLength;
      int realPayloadLength;
      byte[] payload = null;

      appNameLength = in.readUnsignedShort();
      appName = readString(appNameLength);

      appParamCount = in.readUnsignedShort();
      appParams = new String[appParamCount];
      for (int i = 0; i < appParamCount; i++) {
        appParams[i] = readString(in.readUnsignedShort());
      }

      payloadLength = in.readUnsignedShort();
      payload = new byte[payloadLength];
      realPayloadLength = in.read(payload, 0, payloadLength);
      if (realPayloadLength != payloadLength) {
        throw new IOException("Payload length mismatch.");
      }

      return new Packet(appName, appParams, payload);
    } catch (IOException ex) {
      log.error(Localization.getLocalization().getString("M000054"));
    }
    return null;
  }

  /**
   * Reads a String from DataInputStream
   * @param length number of bytes to read
   * @return String containing read bytes
   * @throws java.io.IOException
   */
  String readString(int length) throws IOException {
    String retval;
    byte[] bytes = new byte[length];

    for (int i = 0; i < length; i++) {
      bytes[i] = in.readByte();
    }

    retval = new String(bytes);

    return retval;
  }
}
