/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.incomingQueue;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Queue;

import san.codeDistribution.strategies.CodeDistributionStrategy;
import san.core.CapsuleAddedInQueueListener;
import san.core.Kernel;
import san.data.TransmitCapsule;

import san.localization.Localization;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;

import san.networkInterfaces.RemoteInterface;
import san.networkInterfaces.NetInterfaces;

import san.networkInterfaces.IConnectionChangedListener;
import san.networkInterfaces.IPastReceiving;
import san.networkInterfaces.IInterfacesChanged;
import san.networkInterfaces.NetInterface;
import org.apache.log4j.Logger;
import org.apache.log4j.Logger;
/**
 * <b>Title:</b> Smart Active Node <b>Description:</b> Incoming queue.
 * Contained capsules with data and with applications.
 * @author Jan Boháč, Michal Rejda
 * @since 30.10.2006
 * @version 1.0
 */
public class IncomingQueue
		implements
			IConnectionChangedListener,
			IPastReceiving,
			IInterfacesChanged {
	private static final transient Logger log = Logger.getLogger(IncomingQueue.class);
	
	/**
	 * @uml.property name="kernel"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="inQueue:san.core.Kernel"
	 */
	private Kernel kernel = null;
	/**
	 * @uml.property name="netInterfaces"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="incomingQueue:san.networkInterfaces.NetInterfaces"
	 */
	private NetInterfaces netInterfaces = null;
	private Queue<TransmitCapsule> dataCapsules; // capsules with data
	private Queue<TransmitCapsule> programCapsules; // capsules with programs	
	private Collection<CapsuleAddedInQueueListener> capsuleAddedQueueListeners;
	private CodeDistributionStrategy codeDistributionStrategy;
	
	public IncomingQueue(Kernel kernel, NetInterfaces netInterfaces, CodeDistributionStrategy codeDistributionStrategy) {
		
		this.kernel = kernel;
		this.netInterfaces = netInterfaces;
		this.netInterfaces.addInterfacesChangedListener(this);
		this.codeDistributionStrategy = codeDistributionStrategy;
		
		this.dataCapsules = new LinkedList<TransmitCapsule>();
		this.programCapsules = new LinkedList<TransmitCapsule>();
		this.capsuleAddedQueueListeners = new LinkedList<CapsuleAddedInQueueListener>();
	}

	/**
	 * Get data capsule from queue. Waiting is blocked. If waiting is
	 * interrupted return null.
	 */
	public synchronized TransmitCapsule getCapsule() {
		try {
			while (dataCapsules.size() <= 0) {
				wait();
			}
			notifyAll(); // wake up the other threads
			return dataCapsules.poll(); // return data capsule
		}
		catch (InterruptedException e) {
			log.debug(e);
			return null;
		}
	}

	/**
	 * Get capsule with program code from queue.
	 */
	public synchronized TransmitCapsule getProgramCode() {
		while (programCapsules.size() <= 0) {
			try {
				wait();
			}
			catch (InterruptedException e) {
				log.error(e);
			}
		}
		try {
			return programCapsules.poll(); // return capsule with program
		}
		finally {
			notifyAll(); // wake up the other threads
		}
	}

	public synchronized void received(TransmitCapsule capsule) {
		add(capsule);
	}

	/**
	 * Add the transmitCapsule into the incoming queue and set the over
	 * identifier the same as destination identifier.
	 */
	private void add(TransmitCapsule capsule) {
		
		if(log.isTraceEnabled()) {
			log.trace("add(" + capsule + ")");
		}
		
		fireCapsuleAddedInQueueListeners(capsule);
	
		if(codeDistributionStrategy.shouldCapsuleBeStoredInQueue(capsule)){
			if (log.isDebugEnabled()) {
				log.debug("shouldCapsuleBeStoredInQueue = true");
			}
			
			// add capsule into right queue
			if (capsule.isAplication()) {
				this.programCapsules.add(capsule);
			}
			else if (!capsule.isRequestForApplication()){
				this.dataCapsules.add(capsule);
			}
			notifyAll();
			
		}	
		else{
			if (log.isDebugEnabled()) {
				log.debug("shouldCapsuleBeStoredInQueue = false");
			}
		}
		
		
		codeDistributionStrategy.dealWithCapsuleAfterQueue(capsule);					
		
	}

	public void shutDown() {
		// todo - shut down is not implemented
	}

	/**
	 * Getter of the property <tt>netInterfaces</tt>
	 * @return Returns the netInterfaces.
	 * @uml.property name="netInterfaces"
	 */
	public NetInterfaces getNetInterfaces() {
		return netInterfaces;
	}

	/**
	 * Setter of the property <tt>netInterfaces</tt>
	 * @param netInterfaces The netInterfaces to set.
	 * @uml.property name="netInterfaces"
	 */
	public void setNetInterfaces(NetInterfaces netInterfaces) {
		this.netInterfaces = netInterfaces;
	}

	public void remoteInterfaceAdded(NetInterface localInterface,RemoteInterface remoteInterface) {
		remoteInterface.getNetListener().setPastReceiving(this);
	}

	public void remoteInterfaceLost(NetInterface localInterface,RemoteInterface remoteInterface) {
		// do nothing
	}

	public void interfaceAdded(NetInterface newInterface) {
		newInterface.addConnectionChangedListener(this);
	}

	public void interfaceRemoved(NetInterface removedInterface) {
		removedInterface.removeConnectionChangedListener(this);
	}
	
	private void fireCapsuleAddedInQueueListeners(TransmitCapsule capsule) {
		for (CapsuleAddedInQueueListener l : capsuleAddedQueueListeners) {
			l.capsuleAdded(capsule, IncomingQueue.class.getName());
		}
	}
	
	/**
	 * Add new listener for listen of code storing.
	 */
	public void addCapsuleAddedInQueueListener(CapsuleAddedInQueueListener listener) {
		this.capsuleAddedQueueListeners.add(listener);
	}
}
