/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.networkInterfaces;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.Date;

import org.apache.log4j.Logger;

import san.ident.NetIdentifier;
import san.localization.Localization;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;
import san.networkInterfaces.IncomingPacket.Head;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Class to creating connection betwen two nedes.<br>
 * @author Michal Rejda
 * @since 16.2.2007
 * @version 1.0
 */
public class ConnectionSolver extends Thread {
	
	private static final transient Logger log = Logger.getLogger(ConnectionSolver.class);
	
	/**
	 * Time out by receiving packets.
	 */
	public static final int TIME_OUT = 1000;
	
	private Date lastUsed;	//last used of this connection
	
	/**
	 * @uml.property  name="netInterface"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="connectionSolver:san.networkInterfaces.NetInterface"
	 */
	private NetInterface netInterface = null;
	private DatagramSocket dataSocket;

	/**
	 * last received packet
	 * @uml.property  name="inPacket"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="connectionSolver:san.networkInterfaces.IncomingPacket"
	 */
	private IncomingPacket inPacket = null;
	private boolean running;

	/**
	 * @uml.property  name="previous"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="connectionSolver:san.networkInterfaces.RemoteInterface"
	 */
	private RemoteInterface previous = null;
	
	private NetIdentifier addressTo;	//address of the remote node
	
	/** 
	 * @uml.property name="manager"
	 * @uml.associationEnd multiplicity="(1 1)" inverse="connectionSolvers:san.networkInterfaces.ConnectionsSolverManager"
	 */
	private ConnectionsSolverManager manager = null;
	
	public ConnectionSolver(ConnectionsSolverManager manager, NetInterface netInterface, IncomingPacket inPacket) {
		this.netInterface = netInterface;
		this.manager = manager;
		this.inPacket = inPacket;
		this.addressTo = inPacket.getIdentifier();
		lastUsed = new Date();
	}
	
	public void run() {
		try{
			running = true;
			dataSocket = new DatagramSocket();
			sendResponze(); // STRT
			byte[] data = new byte[IncomingPacket.LENGHT];
			DatagramPacket dp = new DatagramPacket(data,data.length);
			while(running) {
				this.dataSocket.setSoTimeout(TIME_OUT);
				this.dataSocket.receive(dp);
				this.inPacket = new IncomingPacket(dp);
				lastUsed = new Date();
				sendResponze(); // STRT
			}
		} catch(SocketTimeoutException e) {
			;	//not a exception
			running = false;
		} catch(SocketException e){
			LoggerFactory.createLogger().Log(Localization.getLocalization().getString("M000032")+" "+e, LogType.ERROR);
			running = false;
		} catch (IOException e) {
			LoggerFactory.createLogger().Log(Localization.getLocalization().getString("M000033")+" "+e, LogType.ERROR);
			running = false;
		} finally {
			closeConnection();
			removeFromConMgr();
		}
	}
	
	/**
	 * Send response to incoming packet.
	 */
	private void sendResponze() {
		RemoteInterface ri = new RemoteInterface();
		ri.setIdentifier(inPacket.getIdentifier());
		ri.setInetAddress(inPacket.getFromAddress());		// 192.168.134.1
		ri.setCommunicationPort(inPacket.getFromPort());	//55793, 55804, 55799
		ri.setHead(inPacket.getHead());			
		ri.setMethod(inPacket.getMethod());
		ri.setPort(inPacket.getPort());			// 0

		byte[] head = new byte[] {0,0,0,0};//new head
		IncomingPacket.setHeadBit(head, Head.FIND_NEIGHBOURS, true);
		
		if(previous != null) {	//we had received some request from remote node
//			LoggerFactory.createLogger().Log("next prev " + previous.getMethod().number+ " - " + ri.getMethod().number,LogType.DEBUG);
			ri.setConnection(previous.getConnection());
			if(previous.getMethod().number == ri.getMethod().number) {	//the method is set
//				LoggerFactory.createLogger().Log(" == ",LogType.DEBUG);
				if(ri.isSet(Head.ACCEPT) == true) {
					addNewRemoteInterface(ri);
					running = false;
				} else {
					IncomingPacket.setHeadBit(head, Head.ACCEPT, true);	//set the accept of this method of connection
					sendResponze(ri.getInetAddress(), ri.getCommunicationPort(), netInterface.getIdentifier(), ri.getConnection().getLocalPort(), ri.getMethod(),head);
					addNewRemoteInterface(ri);
					running = false;
				}
			} else {
				if(netInterface.isAccessiable(ri.getMethod())) {	//the method is accessible
//					LoggerFactory.createLogger().Log(" accesible ",LogType.DEBUG);
					IncomingPacket.setHeadBit(head, Head.ACCEPT, true);	//set the accept of this method of connection
					sendResponze(ri.getInetAddress(), ri.getCommunicationPort(), netInterface.getIdentifier(), ri.getConnection().getLocalPort(), ri.getMethod(),head);
					addNewRemoteInterface(ri);
					running = false;
				} else {	//the number is't accessible, it must be found another
//					LoggerFactory.createLogger().Log(" finding another",LogType.DEBUG);
					AccessibleMethod am = netInterface.getNextBestMethod(ri.getMethod());
					if(am != null) {	//another was found
						previous.cancelConnection();
						ri.cancelConnection();
						ri.setMethod(am);
						LoggerFactory.createLogger().Log("  previous - " + previous.getMethod().number + " current - " + ri.getMethod().number,LogType.DEBUG);
						ri.createConnection();
						sendResponze(ri.getInetAddress(), ri.getCommunicationPort(), netInterface.getIdentifier(), ri.getConnection().getLocalPort(), ri.getMethod(),head);
						previous = ri;
					} else {	//fall, not accessible method
						Object[] values = new Object[]{
								ri.getIdentifier()
						};
						LoggerFactory.createLogger().Log(Localization.getLocalization().getString("M000034",values),LogType.ERROR);
					}
				}
			}
		} else {	//new incoming request
//			LoggerFactory.createLogger().Log("first",LogType.DEBUG);
			if(netInterface.isAccessiable(ri.getMethod())) {	//the number is accessible
//				LoggerFactory.createLogger().Log(" accesible ",LogType.DEBUG);
				ri.createConnection();
				sendResponze(ri.getInetAddress(), ri.getCommunicationPort(), netInterface.getIdentifier(), ri.getConnection().getLocalPort(), ri.getMethod(),head);
				previous = ri;
			} else {	//the number is not accessible, we must find another
				LoggerFactory.createLogger().Log(" finding another",LogType.DEBUG);
				AccessibleMethod am = netInterface.getNextBestMethod(ri.getMethod());
				if(am != null) {	//another was found
					LoggerFactory.createLogger().Log("  require - " + ri.getMethod().number + " found - " + am.number,LogType.DEBUG);
					ri.setMethod(am);
					ri.createConnection();
					sendResponze(ri.getInetAddress(), ri.getCommunicationPort(), netInterface.getIdentifier(), ri.getConnection().getLocalPort(), ri.getMethod(),head);
					previous = ri;
				} else {	//fall, not accessible method
					Object[] values = new Object[]{
							ri.getIdentifier()
					};
					LoggerFactory.createLogger().Log(Localization.getLocalization().getString("M000034",values),LogType.ERROR);
				}
			}
		}
	}
	
	/**
	 * Sends response to the packet by creating connection with another node.
	 * @param to destination address
	 * @param port destination port
	 * @param thisPort	this port to comunication
	 * @param thisMethod	method to comunication
	 * @param head head of the packet
	 */
	private void sendResponze(InetAddress to, int port, NetIdentifier ident,int thisPort, AccessibleMethod thisMethod, byte[] head) {
		netInterface.sendNighbours(dataSocket, to, port, thisPort, thisMethod, head);
	}
	
	/**
	 * The connection is tied in and this add new remote 
	 * interface and connection to another node.
	 */
	private void addNewRemoteInterface(RemoteInterface ri) {
		if(log.isTraceEnabled()) {
			log.trace("addNewRemoteInterface(\n" + ri.getIdentifier() + ")");
		}
		
		previous = null;	//if will by not null, will be closed
		ri.createInputOutput();	//creating input and output for this remote interface
		netInterface.addRemoteInterface(ri);
	}
	
	/**
	 * Return time latest received packet.
	 */
	public Date getLastUsed() {
		return lastUsed;
	}

	/**
	 * Return identifier of remote node.
	 */
	public NetIdentifier getAddressTo() {
		return addressTo;
	}

	/**
	 * End the connection solver and close all outputs.
	 */
	void closeConnection() {
		this.running = false;
		if(previous != null && previous.getConnection() != null) {
			previous.cancelConnection();
		}
		if (dataSocket != null) {
			dataSocket.close();
		}
	}
	
	/**
	 * Remove this connection solver from connection solver manaker.
	 */
	private void removeFromConMgr() {
		this.manager.solverEnded(this);
	}
}
