/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.networkInterfaces;

import java.net.DatagramPacket;
import java.net.InetAddress;
import san.Utility;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.localization.Localization;

/**
 * <b>Title:</b> Smart Active Node <b>Description:</b> Representation of
 * incoming packet.
 * @author Jan Boháč, Michal Rejda
 * @since 5.12.2006
 * @version 1.0
 */
public class IncomingPacket {

	/**
	 * <b>Title:</b> Small Active Network <b>Description:</b> Enumeration of
	 * features exist in head.
	 * @author Jan Boháč, Michal Rejda
	 * @since 30.11.2006
	 * @version 1.0
	 */
	public enum Head {
		ACCEPT, // the message was accept
		FIND_NEIGHBOURS, // finding neighbours
		FIND_PATH
		// finding path to another node
	}

	/**
	 * Lenght of packet with its header (8+2*Guid.LENGHT bytes).
	 */
	public static final int LENGHT = 8 + 2 * Guid.LENGHT;
	/**
	 * Lenght of packets head (4 bytes).
	 */
	public static final int HEADER_LENGHT = 4;
	private byte[] head;
	private NetIdentifier identifier;
	/**
	 * @uml.property name="method"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="incomingPacket:san.networkInterfaces.AccessibleMethod"
	 */
	private AccessibleMethod method = null;
	private int port;
	private int timeToLive;
	private InetAddress fromAddress;
	private int fromPort;

	/**
	 * Construct incoming packet from datagram packet.
	 */
	public IncomingPacket(DatagramPacket dp) {
		setAttributes(dp);
	}

	/**
	 * Set attributes of this class from the DatagramPacket.
	 */
	private void setAttributes(DatagramPacket dp) {
		fromAddress = dp.getAddress();
		fromPort = dp.getPort();
		byte[] data = dp.getData();
		head = new byte[HEADER_LENGHT]; // setting head
		for (int i = 0; i < HEADER_LENGHT; i++) {
			head[i] = data[i];
		}
		if (isSet(head, Head.FIND_NEIGHBOURS) == true) {
			readFindingNeighboursPacket(data);
		} else if (isSet(head, Head.FIND_PATH) == true) {
			readFindingPathPacket(data);
		}
	}

	private void readFindingNeighboursPacket(byte[] data) {
		byte[] address = new byte[Guid.LENGHT];
		byte[] network = new byte[Guid.LENGHT];
		byte[] tmp;
		for (int i = 4; i < Guid.LENGHT + 4; i++) {// identifier
			address[i - 4] = data[i];
		}
		for (int i = Guid.LENGHT + 4; i < 2 * Guid.LENGHT + 4; i++) {// network
			network[i - 4 - Guid.LENGHT] = data[i];
		}
		this.identifier = new NetIdentifier(new Guid(address),
				new Guid(network));
		tmp = new byte[2];
		tmp[0] = data[2 * Guid.LENGHT + 4];// port
		tmp[1] = data[2 * Guid.LENGHT + 4 + 1];
		port = Utility.array2BytesToInt(tmp);
		tmp[0] = data[2 * Guid.LENGHT + 4 + 2];// method
		tmp[1] = data[2 * Guid.LENGHT + 4 + 3];
		method = new AccessibleMethod(0, Utility.array2BytesToInt(tmp));
	}

	private void readFindingPathPacket(byte[] data) {
		byte[] address = new byte[Guid.LENGHT];
		byte[] network = new byte[Guid.LENGHT];
		byte[] tmp;
		for (int i = 4; i < Guid.LENGHT + 4; i++) {// identifier
			address[i - 4] = data[i];
		}
		for (int i = Guid.LENGHT + 4; i < 2 * Guid.LENGHT + 4; i++) {// network
			network[i - 4 - Guid.LENGHT] = data[i];
		}
		this.identifier = new NetIdentifier(new Guid(address),
				new Guid(network));
		tmp = new byte[2];
		tmp[0] = data[2 * Guid.LENGHT + 4]; // ttl
		tmp[1] = data[2 * Guid.LENGHT + 4 + 1];
		timeToLive = Utility.array2BytesToInt(tmp);
	}

	/**
	 * Set bit of header to 1 od 0 according to true or false value of value.
	 * @param head head array to set
	 * @param headFeature feature in header to set
	 * @param value set the bit to 1 or 0
	 * @return the same array as the head array
	 */
	public static byte[] setHeadBit(byte[] head, Head headFeature, boolean value) {
		switch (headFeature) {
			case FIND_PATH :
				if (!value)
					head[0] = (byte) (Integer.reverse(head[0]) & 0xFF);
				head[0] = (byte) (head[0] | (byte) 0x20);
				if (!value)
					head[0] = (byte) (Integer.reverse(head[0]) & 0xFF);
				break;
			case ACCEPT :
				if (!value)
					head[0] = (byte) (Integer.reverse(head[0]) & 0xFF);
				head[0] = (byte) (head[0] | (byte) 0x40);
				if (!value)
					head[0] = (byte) (Integer.reverse(head[0]) & 0xFF);
				break;
			case FIND_NEIGHBOURS :
				if (!value)
					head[0] = (byte) (Integer.reverse(head[0]) & 0xFF);
				head[0] = (byte) (head[0] | (byte) 0x80);
				if (!value)
					head[0] = (byte) (Integer.reverse(head[0]) & 0xFF);
				break;
			default :
				throw new RuntimeException(Localization.getLocalization()
						.getString("M000040"));
		}
		return head;
	}

	/**
	 * Return a true if a head feature is set.
	 * @param head head to check
	 * @param headFeature head feature
	 * @return true if the feature is set
	 */
	public static boolean isSet(byte[] head, Head headFeature) {
		switch (headFeature) {
			case FIND_PATH :
				return ((head[0] & 0x20) == 0x20);
			case ACCEPT :
				return ((head[0] & 0x40) == 0x40);
			case FIND_NEIGHBOURS :
				return ((head[0] & 0x80) == 0x80);
			default :
				throw new RuntimeException(Localization.getLocalization()
						.getString("M000040"));
		}
	}

	/**
	 * Return true if the packet finding neighbous.
	 */
	public boolean isFindingNeighbouts() {
		return isSet(head, Head.FIND_NEIGHBOURS);
	}

	/**
	 * Return true if the packet finding path into another node.
	 * @deprecated This method is deprecated. Finding node is done into another
	 *             level of server. It is solved on the application level.
	 */
	public boolean isFindingPath() {
		return isSet(head, Head.FIND_PATH);
	}

	/**
	 * @return the fromAddress
	 */
	public InetAddress getFromAddress() {
		return fromAddress;
	}

	/**
	 * @param fromAddress the fromAddress to set
	 */
	public void setFromAddress(InetAddress fromAddress) {
		this.fromAddress = fromAddress;
	}

	/**
	 * @return the fromPort
	 */
	public int getFromPort() {
		return fromPort;
	}

	/**
	 * @param fromPort the fromPort to set
	 */
	public void setFromPort(int fromPort) {
		this.fromPort = fromPort;
	}

	/**
	 * @return the head
	 */
	public byte[] getHead() {
		return head;
	}

	/**
	 * @param head the head to set
	 */
	public void setHead(byte[] head) {
		this.head = head;
	}

	public NetIdentifier getIdentifier() {
		return identifier;
	}

	public void setIdentifier(NetIdentifier address) {
		this.identifier = address;
	}

	/**
	 * @return the port
	 */
	public int getPort() {
		return port;
	}

	/**
	 * @param port the port to set
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * @return the timeToLive
	 */
	public int getTimeToLive() {
		return timeToLive;
	}

	/**
	 * @param timeToLive the timeToLive to set
	 */
	public void setTimeToLive(int timeToLive) {
		this.timeToLive = timeToLive;
	}

	/**
	 * Getter of the property <tt>method</tt>
	 * @return Returns the method.
	 * @uml.property name="method"
	 */
	public AccessibleMethod getMethod() {
		return method;
	}

	/**
	 * Setter of the property <tt>method</tt>
	 * @param method The method to set.
	 * @uml.property name="method"
	 */
	public void setMethod(AccessibleMethod method) {
		this.method = method;
	}
}
