/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.outgoingQueue;

import java.util.Collection;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;

import org.apache.log4j.Logger;

import san.core.CodeStoredListener;
import san.core.CapsuleAddedInQueueListener;
import san.data.TransmitCapsule;
import san.ident.NetIdentifier;
import san.localization.Localization;
import san.logging.ILogger;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;
import san.networkInterfaces.NetOut;
import san.networkInterfaces.NetInterfaces;
import san.networkInterfaces.IConnectionChangedListener;
import san.networkInterfaces.RemoteInterface;
import san.networkInterfaces.IInterfacesChanged;
import san.networkInterfaces.NetInterface;

/**
 * <b>Title:</b> Smart Active Node <br>
 * <b>Description:</b> Output queue.<br>
 * @author Jan Boháč, Michal Rejda
 * @since 30.10.2006
 * @version 1.0
 */
public class OutgoingQueue extends Thread
		implements
			IConnectionChangedListener,
			IInterfacesChanged {
	private static final transient Logger log = Logger
			.getLogger(OutgoingQueue.class);
	/**
	 * Constant to indicate TCP connection.
	 */
	public static final int TCP = 0;
	/**
	 * Constant to indicate UDP connection.
	 */
	public static final int UDP = 1;
	/**
	 * @uml.property name="netInterfaces"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="outgoingQueue:san.networkInterfaces.NetInterfaces"
	 */
	private NetInterfaces netInterfaces = null;
	/**
	 * @uml.property name="netOut"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="outgoingQueue:san.networkInterfaces.NetOut"
	 *                     qualifier="identifier:san.ident.NetIdentifier
	 *                     san.networkInterfaces.NetOut"
	 */
	private Map<NetIdentifier, NetOut> netOutMap;
	private boolean running;
	private Queue<TransmitCapsule> queue;
	private Collection<CapsuleAddedInQueueListener> capsuleAddedQueueListeners = new java.util.LinkedList<CapsuleAddedInQueueListener>();

	private long dataCapsulesSentCount = 0;
	private long appCapsulesSentCount = 0;
	private long appReqCapsulesSentCount = 0;
	
	public OutgoingQueue(NetInterfaces netInterfaces) {
		this.netInterfaces = netInterfaces;
		this.netInterfaces.addInterfacesChangedListener(this);
		this.queue = new LinkedList<TransmitCapsule>();
		this.running = false;
		this.netOutMap = new Hashtable<NetIdentifier, NetOut>();
		this.capsuleAddedQueueListeners = new LinkedList<CapsuleAddedInQueueListener>();
	}

	/**
	 * Add the transmitCapsule into the output queue.
	 */
	public synchronized void add(TransmitCapsule c) {		
		fireCapsuleAddedInQueueListeners(c);
		queue.add(c);		
		notifyAll();
	}

	/**
	 * Return the transmitCapsule from queue.
	 */
	private synchronized TransmitCapsule get() {
		try {
			while (queue.size() <= 0) {
				wait();
			}
			TransmitCapsule capsule = queue.poll();
			if(capsule.isAplication()){
				appCapsulesSentCount++;
			}else if(capsule.isRequestForApplication()){
				appReqCapsulesSentCount++;
			}
			else{
				dataCapsulesSentCount++;
			}
			
			return capsule;
		}
		catch (InterruptedException e) {
			LoggerFactory.createLogger().Log(e, LogType.ERROR);
		}
		return null;
	}

	/**
	 * Translate array of char to byte array.
	 */
	private byte[] toByte(char[] translate) {
		byte[] tmp = new byte[translate.length];
		for (int i = 0; i < tmp.length; i++) {
			tmp[i] = (byte) (translate[i] - '0');
		}
		return tmp;
	}

	public void run() {
		running = true;
		TransmitCapsule c;
		NetOut out = null;

		while (running) {
			c = get(); // get the transmitCapsule from queue

			if (c != null) {
				out = netOutMap.get(c.getOver());
				if (out != null) {
					out.send(c);
//					LoggerFactory.createLogger().Log(this.getClass().getName() + " run(" + c + ") SENT!" , LogType.DEBUG);
					if (log.isDebugEnabled()) {
						log.debug("run(" + c + ") SENT!");
					}
				} else {
					log.error("run(" + c + ") ERROR! no NetOut for 'over' address");					
				}
			}
		}
	}

	/**
	 * @return the running
	 */
	public boolean isRunning() {
		return running;
	}

	/**
	 * Getter of the property <tt>netInterfaces</tt>
	 * @return Returns the netInterfaces.
	 * @uml.property name="netInterfaces"
	 */
	public NetInterfaces getNetInterfaces() {
		return netInterfaces;
	}

	public void remoteInterfaceAdded(NetInterface localInterface,RemoteInterface remoteInterface) {
		netOutMap.put(remoteInterface.getIdentifier(), remoteInterface
				.getNetOut());
	}

	public void remoteInterfaceLost(NetInterface localInterface,RemoteInterface remoteInterface) {
		netOutMap.remove(remoteInterface.getIdentifier());
	}

	public void interfaceAdded(NetInterface newInterface) {
		newInterface.addConnectionChangedListener(this);
	}

	public void interfaceRemoved(NetInterface removedInterface) {
		removedInterface.removeConnectionChangedListener(this);
	}
	
	private void fireCapsuleAddedInQueueListeners(TransmitCapsule capsule) {
		for (CapsuleAddedInQueueListener l : capsuleAddedQueueListeners) {
			l.capsuleAdded(capsule, OutgoingQueue.class.getName());
		}
	}
	
	/**
	 * Add new listener for listen of code storing.
	 */
	public void addCapsuleAddedInQueueListener(CapsuleAddedInQueueListener listener) {
		this.capsuleAddedQueueListeners.add(listener);
	}
	
	public long getDataCapsulesSentCount() {
		return dataCapsulesSentCount;
	}

	public long getAppCapsulesSentCount() {
		return appCapsulesSentCount;
	}

	public long getAppReqCapsulesSentCount() {
		return appReqCapsulesSentCount;
	}
}
